<?
//////////////////////////////////////////////////////////////////////////
//	"pgsql,inc" for SDB Tools
//------------------------------------------------------------------------
//	$Id: pgsql.inc 165 2006-05-06 01:21:59Z taro $
//////////////////////////////////////////////////////////////////////////

class	PgRid	// Result ID
{
	var		$id;

	function PgRid()
	{
		$this->id = FALSE;
	}

	function Init()
	{
		$this->Free();
	}
	function Term()
	{
		$this->Free();
	}

	function Free()
	{
		if ( $this->id )
		{
			pg_freeresult($this->id);
			$this->id = FALSE;
		}
	}
	function GetNumFields()
	{
		if ( ! $this->id )
			return -1;
		return pg_numfields($this->id);
	}
	function GetNumRows()
	{
		if ( ! $this->id )
			return -1;
		return pg_numrows($this->id);
	}
	function	GetFieldName($fno)
	{
		if ( ! $this->id )
			return -1;
		return pg_fieldname($this->id, $fno);
	}
	function	GetResult($row, $filed)
	{
		if ( ! $this->id )
			return -1;
		return pg_result($this->id, $row, $filed);
	}
	function	GetLastOID()
	{
		if ( ! $this->id )
			return -1;
		return pg_getlastoid($this->id);
	}
}

class	PgSql
{
	var		$conn;	// connection index
	var		$host;
	var		$user;
	var		$password;
	var		$port;
	var		$option;
	var		$tty;
	var		$dbname;

	var		$work_level;
	var		$work_rollback;

	var		$debug;

	function	PgSql()
	{
		$this->debug = 0;
		$this->conn = FALSE;
		$this->work_level = 0;
		$this->work_rollback = FALSE;
	}

	// 
	function	Init()
	{
		if ( $this->conn )
			$this->Close();
		$this->host     = "localhost";
		$this->user     = "";
		$this->password = "";
		$this->port     = 5432;
		$this->option   = "";
		$this->tty      = "";
		$this->dbname   = "";
	}
	function Term()
	{
		$this->Close();
	}

	function	Connect($persistent)
	{
		if ( $this->conn || $this->host == "" || $this->dbname == "" )
			return -1;
		$param = "";
		if ( $this->user != "" )
		{
			$param = "host=" . $this->host;
			if ( $this->user )
				$param .= " user=" . $this->user;
			if ( $this->password )
				$param .= " password=" . $this->password;
			if ( $this->port )
				$param .= " port=" . $this->port;
			if ( $this->option )
				$param .= " options=" . $this->option;
			if ( $this->tty )
				$param .= " tty=" . $this->tty;
			if ( $this->dbname )
				$param .= " dbname=" . $this->dbname;
		}
		if ( $persistent )
		{
			if ( $param )
				$this->conn = pg_pconnect($param);
			else
				$this->conn = pg_pconnect( $host, $port, $option, $tty, $dname);
		} else
		{
			if ( $param )
				$this->conn = pg_connect($param);
			else
				$this->conn = pg_connect( $host, $port, $option, $tty, $dname);
		}
		if ( $this->conn == FALSE )
			return -1;
		return 0;
	}

	function	Close()
	{
		if ( $this->conn )
		{
			if ( $this->work_level > 0 )
			{
				$this->work_level = 1;
				$this->CommitWork();
			}
			pg_close ($this->conn);
		}
		$conn = FALSE;
	}

	function	Exec($param)
	{
		if ( ! $this->conn )
			return -1;
	//	echo "DEBUG pgsql Exec -- $param<BR>\n";
		$rid = new PgRid;
		$rid->id = pg_exec($this->conn, $param);
		if ( $rid->id == FALSE )
			return -1;
		return $rid;
	}

	function Select($param)
	{
		if ( ! $this->conn )
			return -1;
		$param = "SELECT " . $param;
		return $this->Exec($param);
	}

	function	BeginWork()
	{
		if ( ! $this->conn )
			return -1;
		if ( $this->work_level == 0 )
		{
			$this->work_rollback = FALSE;
			if ( $this->Exec("BEGIN WORK") < 0 )
				return -1;
		}
		$this->work_level++;
		return 0;
	}
	function	CommitWork()
	{
		if ( ! $this->conn )
			return -1;
		if ( $this->work_level > 0 )
		{
			$this->work_level--;
			if ( $this->work_level == 0 )
			{
				if ( $this->work_rollback == FALSE )
				{
					if ( $this->Exec("COMMIT WORK") < 0 )
						return -1;
				} else
				{
					if ( $this->Exec("ROLLBACK WORK") < 0 )
						return -1;
				}
			}
		}
		return 0;
	}
	function	RollbackWork()
	{
		if ( ! $this->conn )
			return -1;
		if ( $this->work_level > 0 )
		{
			$this->work_level--;
			if ( $this->work_level == 0  )
			{
				if ( $this->Exec("ROLLBACK WORK") < 0 )
					return -1;
			} else
				$this->work_rollback = TRUE;	// BugFix 2005.01.01
		}
		return 0;
	}

	function	loCreate()
	{
	//	if ( ! $this->conn )
	//		return -1;
		return pg_locreate($this->conn);	// return oid
	}
	function	loOpen($oid, $mode)
	{
	//	if ( ! $this->conn )
	//		return -1;
	//	echo "oid = $oid<BR>";
	//	echo "mode = $mode<BR>";
		return pg_loopen($this->conn, (int)$oid, $mode);
	}
	function	loClose($fd)
	{
		return pg_loclose($fd);
	}
	function	loWrite($fd, $buf)
	{
		return pg_lowrite($fd, $buf);
	}
	function	loRead($fd, $len)
	{
		return pg_loread($fd, $len);
	}
	function	loReadAll($fd)
	{
		return pg_loreadall($fd);
	}
	function	loUnlink($oid)
	{
		return pg_lounlink($this->conn, $oid);
	}

	function	EncodeStringOld($s)
	{
		$rpl_tbl = array( 
			array("FROM"=>"'" , "TO"=>"''"   ),
			array("FROM"=>"\\", "TO"=>"\\\\" )
		);
		for ( $i = 0; $i < count($rpl_tbl); $i++ )
			$s = str_replace( $rpl_tbl[$i]["FROM"], $rpl_tbl[$i]["TO"], $s);
		return "'{$s}'";
	}
	function	EncodeString($s)
	{
		if ( ! ereg("[\x01-\x1F\x5C\x7F-\xFF]", $s) )	// [\x01-0x1F \ 0x7F-0xFF]
		{
		//	echo "ASCII<BR>";	// DEBUG
			return "'".str_replace( "'", "''", $s)."'";
		}
		$dst = "'";
		$s_len = strlen($s);
		$pos = 0;
		while ( $pos < $s_len )
		{
			$ch = substr($s, $pos, 1);
			$code = ord($ch);
			$pos++;
			if ( $code < 0x80 )
			{	// 00-7F
				switch ( $code )
				{
				case 0x08:	// '\b'
					$dst .= "\\b";
					break;
				case 0x09:	// '\t':
					$dst .= "\\t";
					break;
				case 0x0A:	// '\n'
					$dst .= "\\n";
					break;
				case 0x0B:	// '\v'
					$dst .= "\\v";
					break;
				case 0x0C:	// '\f'
					$dst .= "\\f";
					break;
				case 0x0D:	// '\r'
					$dst .= "\\r";
					break;

				case 0x27:	// '
					$dst .= "''";
					break;
				case 0x5C:	// '\'
					$dst .= "\\\\";
					break;

				default:
					if ( $code < 0x20 || $ch == 0x7F )
						$dst .= sprintf("\\%03o", $code);
					else
						$dst .= $ch;
					break;
				}
			} else
			{	// Multi-Byte
				$ch2 = substr($s, $pos, 1);
				$code2 = ord($ch2);
				if ( $code > 0xA0 && $code <= 0xFF && $code2 >= 0xA0 && $code2 <= 0xFF )
				{	// 2bytes EUC
					$dst .= $ch;
					$dst .= $ch2;
					$pos++;
				} else if ( $code == 0x8E && $code2 >= 0xA0 && $code2 <= 0xFF )
				{	// 2bytes EUC Kana
					$dst .= $ch;
					$dst .= $ch2;
					$pos++;
				} else
				{
					$ch3 = substr($s, $pos + 1, 1);
					$code3 = ord($ch3);
					if ( $code == 0x8F && $code2 >= 0xA0 && $code2 <= 0xFF && $code3 >= 0xA0 && $code3 <= 0xFF)
					{	// 3bytes EUC
						$dst .= $ch;
						$dst .= $ch2;
						$dst .= $ch3;
						$pos += 2;
					} else
					{	// ERROR
						$dst .= sprintf("\\%03o", $code);
					}
				}
			}
		}
		$dst .= "'";
		return $dst;
	}

	function	GetTsParts($ts)
	{
		if ( ereg( "([0-9]{4})-([0-9]{1,2})-([0-9]{1,2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})", $ts, $a ) )
		{
			$ret["year"  ] = $a[1];
			$ret["month" ] = $a[2];
			$ret["day"   ] = $a[3];
			$ret["hour"  ] = $a[4];
			$ret["minute"] = $a[5];
			$ret["second"] = $a[6];
			for ( $i = 0; $i < count($a) - 1; $i++ )
				$ret[$i] = (int)$a[$i+1];
			return $ret;
		} else
			return -1;
	}

	function	lib_CreateTable($tbl_name, $tbl_def, $addtbl = "")
	{
		$idx_n = 0;
		$sql = "CREATE TABLE {$tbl_name} (\n";
		$primary_keys = "";
		$ks = array_keys($tbl_def);
		for ( $i = 0; $i < count($ks); $i++ )
		{
			if ( $i > 0 )
				$sql .= ",\n";
			$column = $ks[$i];
			$type = $tbl_def[$column]["TYPE"];
			if ( $tbl_def[$column]["UNIQUE"] )
				$type .= " UNIQUE";
			if ( $tbl_def[$column]["PRIMARY"] )
			{
				if ( $primary_keys != "" )
					$primary_keys .= ",";
				$primary_keys .= $column;
			}
			if ( $tbl_def[$column]["INDEX"] )
			{	// ǥå
				$idx_sql[$idx_n] = "CREATE INDEX {$tbl_name}_{$column}_idx ON {$tbl_name} ({$column})";
				$idx_n++;
			}
			$sql .= "{$column} {$type}";
		}
		if ( $primary_keys != "" )
			$sql .= "\n,PRIMARY KEY(${primary_keys})";
		if ( $addtbl != NULL )
			$sql .= "\n,{$addtbl}";
		$sql .= ")\n";
		if ( $this->debug )
			echo "pgsql->lib_CreateTbale(): sql = $sql<BR>\n";
		$ret = $this->Exec($sql);
		if ( $ret < 0 )
			return -1;
		$ret->Term();

		if ( $idx_n > 0 )
		{	// ǥåκ
			for ( $i = 0; $i < $idx_n; $i++ )
			{
				$sql = $idx_sql[$i];
				if ( $this->debug )
					echo "pgsql->lib_CreateTbale(): sql = $sql<BR>\n";
				$ret = $this->Exec($sql);
				if ( $ret < 0 )
					return -1;
			}
		}
		return 0;
	}

	function	lib_DropTable($tbl_name, $tbl_def)
	{
		$idx_n = 0;
		$sql = "DROP TABLE {$tbl_name}\n";
		$this->Exec($sql);

		$ks = array_keys($tbl_def);
		for ( $i = 0; $i < count($ks); $i++ )
		{
			$column = $ks[$i];
			$type = $tbl_def[$column]["TYPE"];
			if ( $type == "serial" )
			{
				$sql = "DROP SEQUENCE {$tbl_name}_{$column}_seq";
				if ( $this->debug )
					echo "pgsql->lib_DropTbale(): sql = $sql<BR>\n";
				$this->Exec($sql);
			}
		/*	if ( $tbl_def[$column]["INDEX"] )
			{
				$sql = "DROP INDEX {$tbl_name}_{$column}_idx";
				if ( $this->debug )
					echo "pgsql->lib_DropTbale(): sql = $sql<BR>\n";
				$this->Exec($sql);
			}
		*/
		}
	}

	// SQL INSERT μ
	function	lib_GetSqlInsert($tbl_name, $tbl_def, &$data)
	{
		$flds = "";
		$vals = "";
		$ks = array_keys($tbl_def);
		for ( $i = 0; $i < count($ks); $i++ )
		{
			$k = $ks[$i];
			if ( ! isset($data[$k]) )	// ǡǼƤʤ
				continue;
			$v = $data[$k];
			$type = $tbl_def[$k]["TYPE"];
			if ( substr($type, 0, 6) == "serial" )
			{
				continue;
			} elseif ( substr($type, 0, 4) == "text" || substr($type, 0, 7) == "varchar" )
			{	// ʸ
				$v = $this->EncodeString((string)$v);
			} elseif ( $type == "bool" )
			{
				$v = ($v == TRUE || substr($v,0,1) == 't') ? 'true' : 'false';
			} elseif ( $type == "date" || $type == "time" || $type == "timestamp" )
			{	// ա
				if ( $v == "" )
					continue;
				$v = "'{$v}'";
			} elseif ( substr($type, 0, 3) == "int" || substr($type, 0, 5) == "float" )
			{	// 
				if ( (string)$v == "" )
					continue;
			}

			if ( $flds != "" )
				$flds .= ",";
			$flds .= $k;

			if ( $vals != "" )
				$vals .= ",";
			$vals .= $v;
		} // end of for(;;)
		if ( $flds == "" || $vals == "" )
			return "";
		$sql = "INSERT INTO {$tbl_name} ($flds) VALUES ($vals)";
		return $sql;
	}
	// SQL ¹Ԥ ODI 
	function	lib_ExecSqlInsert($sql)
	{
		if ( ($ret = $this->Exec($sql)) < 0 )
			return -1;	// error
		$oid = $ret->GetLastOID();
		$ret->Term();
		return $oid;
	}

	// SQL UPDATE μ
	function	lib_GetSqlUpdate($tbl_name, $tbl_def, &$data, $where)
	{
		$sets = "";
		$ks = array_keys($tbl_def);
		for ( $i = 0; $i < count($ks); $i++ )
		{
			$k = $ks[$i];
			if ( ! isset($data[$k]) )	// ǡǼƤʤ
				continue;
			$v = $data[$k];
			$type = $tbl_def[$k]["TYPE"];
			if ( $type == "serial" )
			{
				continue;
			} elseif ( substr($type, 0, 4) == "text" || substr($type, 0, 7) == "varchar" )
			{	// ʸ
				$v = $this->EncodeString((string)$v);
			} elseif ( $type == "bool" )
			{
				$v = ($v == TRUE || substr($v,0,1) == 't') ? 'true' : 'false';
			} elseif ( $type == "date" || $type == "time" || $type == "timestamp" )
			{	// ա
				$v = $v != "" ? "'{$v}'" : "NULL";
			} elseif ( substr($type, 0, 3) == "int" || substr($type, 0, 5) == "float" )
			{	// 
				$v = (string)$v != "" ? $v : "NULL";
			}

			if ( $sets != "" )
				$sets .= ",";
			$sets .= "{$k}={$v}";
		} // end of for(;;)
		if ( $sets == "" )
			return "";
		$sql = "UPDATE {$tbl_name} SET {$sets}";
		if ( $where != "" )
			$sql .= " WHERE {$where}";
		return $sql;
	}

	function	lib_SelectSql(&$list, $sql, $tbl_def)
	{
		$ret = $this->Exec($sql);
		if ( $ret < 0 )
			return -1;
		$y_max = $ret->GetNumRows();
		$x_max = $ret->GetNumFields();
		for ( $y = 0; $y < $y_max; $y++ )
		{
			for ( $x = 0; $x < $x_max; $x++ )
			{
				$f = $ret->GetFieldName($x);
				$list[$y][$f] = $ret->GetResult($y, $x);
				if ( $tbl_def[$f]["TYPE"] == "bool" )
					$list[$y][$f] = (substr($list[$y][$f],0,1) == 't') ? TRUE : FALSE;
			}
		}
		$ret->Term();
		return $y_max;
	}

	function	lib_CsvSave($opt, $path, $sql, $tbl_def)
	{
		$ret = $this->Exec($sql);
		if ( $ret < 0 )
			return -1;
		$y_max = $ret->GetNumRows();
		$x_max = $ret->GetNumFields();
		if ( $y_max == 0 || $x_max == 0 )
		{
			$retcode = 0;
		} else
		{
			if ( ($fp = fopen($path,"w")) == FALSE )
			{	// error
				$retcode = -1;
			} else
			{
				if ( $opt["FIELD"] )
				{	// ե̾ν
					$buffer = "";
					for ( $x = 0; $x < $x_max; $x++ )
					{
						$f = $ret->GetFieldName($x);
						if ( $buffer != "" )
							$buffer .= ",";
						$buffer .= $this->csv_encode($f);
					}
					$buffer .= "\n";
					fputs($fp, $buffer);
				}
				for ( $y = 0; $y < $y_max; $y++ )
				{
					unset($data);
					for ( $x = 0; $x < $x_max; $x++ )
					{
						$f = $ret->GetFieldName($x);
						$data[$f] = $ret->GetResult($y, $x);
					}
					$buffer = $this->lib_CsvCnvData(&$data, $tbl_def);
					fputs($fp, $buffer);
				}
				fclose($fp);
				$retcode = $y_max;
			}
		}
		$ret->Term();
		return $retcode;
	}

	function	lib_CsvCnvData(&$data, $tbl_def)
	{
		$buffer = "";
		$ks = array_keys($tbl_def);
		for ( $i = 0; $i < count($ks); $i++ )
		{
			$f = $ks[$i];
			$type = $tbl_def[$f]["TYPE"];
			$v = $data[$f];
			if ( $i > 0 /*buffer != ""*/ )
				$buffer .= ",";
			if ( substr($type, 0, 4) == "text" || substr($type, 0, 7) == "varchar" )
				$v = $this->csv_encode($v);
			$buffer .= $v;
		}
		$buffer .= "\n";
		return $buffer;
	}

	function	csv_encode($s)
	{
		$buffer = '"';
		$len = strlen($s);
		for ( $pos = 0; $pos < $len; ++$pos )
		{
			$ch = substr($s, $pos, 1);
			if ( $ch == '"' )
				$ch = '""';
			elseif ( $ch == "\r" )
				$ch = '';
			$buffer .= $ch;
		}
		$buffer .= '"';
		return $buffer;
	}

	function	lib_CsvGetData($opt, $fp, &$data)
	{
		if ( feof($fp) )
			return 0;
		$mode = "top";
		$n = 0;
		$buffer = fgets($fp, 4096);
		for ( $pos = 0; $pos < strlen($buffer); $pos++ )
		{
			$ch = substr($buffer, $pos, 1);

			switch ( $mode )
			{
				case "top":
					switch ( $ch )
					{
						case ' ':
						case "\n":
						case "\r":
							break;
						case '"':
							$mode = "str";
							break;
						case ',':
							$n++;
							break;
						default:
							$mode = "data";
							$data[$n] .= $ch;
							break;
					}
					break;

				case "data":
					switch ( $ch )
					{
						case "\n":
						case "\r":
							break;
						case ',':
							$n++;
							$mode = "top";
							break;
						default:
							$data[$n] .= $ch;
							break;
					}
					break;

				case "str":
					if ( $ch == '"' )
					{
						if ( substr($buffer, $pos+1, 1) == '"' )
						{
							$pos++;
							$data[$n] .= '"';
						} else
						{
							$n++;
							$mode = "end";
						}
					} else
					{
						if ( $ch == "\r" )
							break;
						if ( $ch == "\n" )
							$ch = "\r\n";
						$data[$n] .= $ch;
						if ( $pos + 1 == strlen($buffer) )
						{
							$buffer = fgets($fp, 4096);
							$pos = -1;
						}
					}
					break;

				case "end":
					if ( $ch == ',' )
						$mode = "top";
					break;
			}
		}
		if ( $mode == "data" || $mode == "str" )
			$n++;
		return $n;
	}

} // end of "calss PgSql"

?>
