//////////////////////////////////////////////////////////////////////////
//	"sdblib.h"
//------------------------------------------------------------------------
//	$Id: sdblib.h 161 2006-05-05 10:39:26Z taro $
//////////////////////////////////////////////////////////////////////////
#ifndef	SDBLIB_H_INCLUDED
#define	SDBLIB_H_INCLUDED

#include	"sdb_defs.h"
#include	"sdb_tbldef.h"
#include	"../lib/sqllib/sqllib.h"
#if	defined(WIN32)
# include	"../win32/devctl.h"
#elif	defined(LINUX)
# include	"../linux/devctl.h"
#endif
#if	defined(SDB_USE_SQLIB_SQSTR)
# include	<sqstr.h>
#else
# include	"../lib/sqstr.h"
#endif
#if	defined(LINUX)
# include	<stdio.h>
#endif
#include	<list>

#define	SDB_GENERAL_VERSION	TEXT("Version 0.99a edition #01")
#define	SDB_DEFFILENAME		TEXT("sdb.def")

#define	SDB_DEFAULT_HOST	TEXT("localhost")
#define	SDB_DEFAULT_PORT	TEXT("5432")
#define	SDB_DEFAULT_USER	TEXT("sdb_user")
#define	SDB_DEFAULT_PASS	TEXT("sdb_pass")
#define	SDB_DEFAULT_DBNAME	TEXT("sdb")

#define	SDB_DEV_TBLNAME			TEXT("dev")
#define	SDB_DEFAULT_SMT_TBLNAME	TEXT("smt")
#define	SDB_WAT_TBLNAME			TEXT("wat")
#if	defined(LINUX) && !defined(SDB_LOADDEF_DIR)
#	define	SDB_LOADDEF_DIR		TEXT("/var/adm/tools")
#endif

#define	SDB_LOG_FILENAME		TEXT("smart.log")

#define	SDB_SMART_ATTR_ID_MAX	(256)

#define	SDB_RAW_WORD_MAX			(65536)
#define	SDB_RAW_WORD_MARGIN			(4320)	// 24H ~ 3day
#define	SDB_RANDOM_WAIT_SEC			(30)
#define	SDB_DIALOG_AUTO_CLOSE_SEC	(15)


#if	defined(SDB_USE_DEVCTL)
class	DevCtl;
#endif
class	GetOpt;
class	SdbLib;
class	SdbDev;
class	ScrLib;
class	VarBuffer;

namespace	sdb
{
	struct DBInfo
	{
		SQStr		host;
		SQStr		port;
		SQStr		user;
		SQStr		pass;
		SQStr		dbname;
	};
	extern const TCHAR				*db_host;
	extern const TCHAR				*db_port;
	extern const TCHAR				*db_user;
	extern const TCHAR				*db_pass;
	extern const TCHAR				*db_name;

	extern const TCHAR				*sub_dir;

	extern const TCHAR				*http_post_host;
	extern const TCHAR				*http_post_port;
	extern const TCHAR				*http_post_path;
	extern const TCHAR				*http_auth_type;
	extern const TCHAR				*http_auth_user;
	extern const TCHAR				*http_auth_pass;

	extern int						mail_warnlevel;
	extern const TCHAR				*mail_from;
	extern const TCHAR				*mail_to;
	extern const TCHAR				*mail_smtp_server;

	extern int						warn_high_estimate;
	extern int						warn_mid_estimate;
	extern int						warn_low_estimate;
	extern int						watch_offline_margin_sec;

	extern int						auto_raw_word_id;		// for sdbwatch
	extern int						random_wait_sec;		// for sdbtask
	extern int						dialog_auto_close_sec;	// for sdbtask

	struct	DBDevData
	{
		unsigned			dev_id;
		SQStr				dev_model;
		SQStr				dev_serial;
		SQStr				dev_hostname;
		SQStr				dev_groupname;
		SQStr				dev_smt_tblname;
		BOOL				dev_f_disable;
		unsigned			dev_blockSize;
		Int64				dev_blockNumber;
		SqlLib::TimeStamp	dev_ts_begin;
		SqlLib::TimeStamp	dev_ts_last;
		unsigned			dev_user_stt;
		unsigned			dev_user_id;
		SQStr				dev_user_pass;
		SqlLib::TimeStamp	dev_user_ts;
		SQStr				dev_memo;

		DBDevData();
		DBDevData(const DBDevData & src);
		DBDevData & 		operator = (const DBDevData & src);
	};
	struct	DBSmtData
	{
		SqlLib::TimeStamp	smt_ts;
		int					smt_dev_id;
		unsigned			smt_attr_id;
		unsigned			smt_value;
		unsigned			smt_worst;
		unsigned			smt_threshold;
		unsigned			smt_flags;
		Int64				smt_raw_value;
		BYTE				smt_raw_data[6];

		DBSmtData();
		DBSmtData(const DBSmtData & src);
		DBSmtData & 		operator = (const DBSmtData & src);
	};

	enum	WatchType
	{
		WATCHTYPE_OFF          = 0,
		WATCHTYPE_ATTR_ANALYS  = 1,
		WATCHTYPE_ATTR_VALUE   = 2,
		WATCHTYPE_RAW_VALUE    = 3,
		WATCHTYPE_RAW_ANALYS   = 4,	// RAW VALUE̗\
		WATCHTYPE_TIMESTAMP    = 5,
		WATCHTYPE_NUM,
	};
	enum	WatchWarn
	{
		WATCHWARN_OFF          = 0,
		WATCHWARN_FAILURE      = 1,
		WATCHWARN_HIGH         = 2,
		WATCHWARN_MID          = 3,
		WATCHWARN_LOW          = 4,
		WATCHWARN_NUM,
	};
	struct	DBWatData
	{
		unsigned			wat_id;
		int					wat_priority;
		SQStr				wat_model;
		SQStr				wat_serial;
		unsigned			wat_attr_id;
		WatchType			wat_type;
		BOOL				wat_f_disable;
		unsigned			wat_warn;
		SQStr				wat_op;
		SQStr				wat_right;

		DBWatData();
		DBWatData(const DBWatData & src);
		DBWatData & 		operator = (const DBWatData & src);
	};

	typedef	std::list<DBDevData>	DBDevDataList;


	class DBTblLib
	{
	friend class	SdbLib;
	protected:
		SdbLib				*sdblib;

	public:
		SQStr				tblname;
		SqlLib::TableDef	tbldef;

	public:
		DBTblLib();
		virtual ~DBTblLib();
		SdbLib *	GetSdbLib(void) const { return this->sdblib; }
		SqlLib *	GetSqlLib(void) const;
		int			CreateTable(const TCHAR * tblbane = NULL, const TCHAR * addtbl = NULL);
		int			DropTable(const TCHAR * tblname = NULL);
		SqlLib::Oid	InsertData(SqlLib::FieldsTbl & data);
		int			UpdateData(SqlLib::FieldsTbl & data, const TCHAR * where);
		int			SelectData(SqlLib::Result & sqlres, const TCHAR * query);
		int			DeleteData(const TCHAR * where);
	};

	class	DBDevLib: public DBTblLib
	{
	public:
		DBDevLib();
		~DBDevLib();

		int				CreateTable(void);
		int				SearchDevice(SqlLib::FieldsTbl & data, const TCHAR * model, const TCHAR * serial);
		int				SearchDevice(DBDevData & data, const TCHAR * model, const TCHAR * serial);
		int				GetDevData(DBDevData & data, SqlLib::FieldsTbl & fdata);
		int				SetDevData(SqlLib::FieldsTbl & fdata, const DBDevData & data);
		static BOOL		GetDataValue(SqlLib::DataValue & val, SqlLib::FieldsTbl & data, DevFldNo no);
		int				AddDevice(SqlLib::FieldsTbl & fdata, const DBDevData & data);
		int				DeleteDevice(const DBDevData & devdata);
		int				GetDevDataList(DBDevDataList & list, const TCHAR * add_sql = NULL);
	};

	class	DBSmtLib: public DBTblLib
	{
	public:
		DBSmtLib();
		~DBSmtLib();

		int				GetSmtData(DBSmtData & data, SqlLib::Result & sqlres, int y, int xmax = 0, const TCHAR ** fnametbl = NULL);
		int				GetSmtData(DBSmtData & data, SqlLib::FieldsTbl & fdata);
		int				SetSmtData(SqlLib::FieldsTbl & fdata, const DBSmtData & data);
		int				CreateTable(void);
	};

	class	DBWatLib: public DBTblLib
	{
	public:
		DBWatLib();
		~DBWatLib();

		static DBWatLib *	Create(SdbLib * sdblib);
		int				GetWatData(DBWatData & data, SqlLib::FieldsTbl & fdata);
		int				SetWatData(SqlLib::FieldsTbl & fdata, const DBWatData & data);
		int				CreateTable(void);
	};

	class	DeviceOption
	{
	public:
		SQStr		model;
		SQStr		serial;
		SQStr		option;
	};

	//////////////////////////////////////////////////////////////////////
	//	S.M.A.R.T.

	enum	SdbLibEnum
	{
		SMART_ATTRIBUTE_MAX_LENGHTH = 30,
	};
	enum	SmartVendor
	{
		SMART_VENDOR_GENERAL,
		SMART_VENDOR_FUJITSU,
		SMART_VENDOR_HGST   ,
		SMART_VENDOR_IBM    ,
		SMART_VENDOR_MAXTOR ,
		SMART_VENDOR_SAMSUNG,
		SMART_VENDOR_SEAGATE,
		SMART_VENDOR_TOSHIBA,
		SMART_VENDOR_WD     ,
	};
	// SMART Attributes List
	enum	SmartAttrRawType
	{
		SMART_RAW_NORMAL ,
		SMART_RAW_DUMPHEX,
		SMART_RAW_DIVWORD,
		SMART_RAW_WORD0,
		SMART_RAW_WORD1,
		SMART_RAW_WORD2,
	};
	struct SmartAttrItem
	{
		unsigned			id;
		SmartAttrRawType	type;
		SQStr				text;
	};
	class	SmartAttrList
	{
	public:
		typedef std::list<SmartAttrItem *>	List;
		List		list;

		SmartAttrList();
		~SmartAttrList();
		void			clear(void);
		void			push_back(SmartAttrItem * item);
		int				AddDef(const TCHAR * param);
		SmartAttrItem *	GetSmartAttrItem(unsigned id);
		const TCHAR * 	GetSmartRawValue(DevCtl::AtaSmartAttribute & attr, unsigned length, TCHAR * buffer);
		Int64			GetSmartRawValue(DevCtl::AtaSmartAttribute & attr);
	};
	extern const TCHAR * GetSmartAttributeIdName(SmartVendor vendor, unsigned id, unsigned length = 0, TCHAR * buffer = NULL);

	extern inline Uint64		ATA_GET48(void *p)	{ return (
		 ((Uint64)((BYTE *)(p))[0])
		+((Uint64)((BYTE *)(p))[1]<< 8)
		+((Uint64)((BYTE *)(p))[2]<<16)
		+((Uint64)((BYTE *)(p))[3]<<24)
		+((Uint64)((BYTE *)(p))[4]<<32)
		+((Uint64)((BYTE *)(p))[5]<<40)
	); }
	extern inline DWORD		ATA_GET32(void *p)	{ return (
		 ((DWORD)((BYTE *)(p))[0])
		+((DWORD)((BYTE *)(p))[1]<< 8)
		+((DWORD)((BYTE *)(p))[2]<<16)
		+((DWORD)((BYTE *)(p))[3]<<24)
	); }
	extern inline unsigned		ATA_GET16(void *p)	{ return (
		 ((unsigned)((BYTE *)(p))[0])
		+((unsigned)((BYTE *)(p))[1]<< 8)
	); }
	extern inline void		ATA_SET48(void *p, Uint64 n)
	{
		((BYTE *)(p))[0] = (BYTE)( n      & 0xFF);
		((BYTE *)(p))[1] = (BYTE)((n>> 8) & 0xFF);
		((BYTE *)(p))[2] = (BYTE)((n>>16) & 0xFF);
		((BYTE *)(p))[3] = (BYTE)((n>>24) & 0xFF);
		((BYTE *)(p))[4] = (BYTE)((n>>32) & 0xFF);
		((BYTE *)(p))[5] = (BYTE)((n>>40) & 0xFF);
	}

	//////////////////////////////////////////////////////////////////////
	// DATE/TIME
	extern int		GetSystemTime(SqlLib::TimeStamp & ts);
	extern int		GetLocalTime(SqlLib::TimeStamp & lt, const SqlLib::TimeStamp & st);
	extern int		CompTimeStamp(const SqlLib::TimeStamp & t1, const SqlLib::TimeStamp & t2);
	extern int		OffsetTimeStamp(SqlLib::TimeStamp & ts, const SqlLib::TimeStamp & src, int sec);
	extern int		DiffTimeStamp(const SqlLib::TimeStamp & t1, const SqlLib::TimeStamp & t2);

	extern TCHAR *	Fmt(TCHAR * buffer, const TCHAR * form, ... );
	extern int		CorrectValueName(SQStr & src);

	//////////////////////////////////////////////////////////////////////
	//	FILE I/O
	enum	FileOpenMode
	{
		FOM_NOTHING ,
		FOM_READ    ,
		FOM_WRITE   ,
		FOM_APPEND  ,
	};

#if	defined(WIN32)
	typedef	HANDLE		FileHandle;
#elif	defined(LINUX)
	typedef int			FileHandle;
#else
	typedef	FILE *		FileHandle;
#endif
	extern const FileHandle	Invalid_FileHandle_Value;
	extern BOOL			IsFileHandleValid(FileHandle handle);
	extern FileHandle	FileOpen(const TCHAR *path, FileOpenMode mode);
	extern int			FileClose(FileHandle handle);
	extern int			FileRead(FileHandle handle, unsigned size, void * buffer);
	extern int			FileWrite(FileHandle handle, unsigned size, const void * buffer);
	extern Int64		FileSeek(FileHandle handle, Int64 offset, int pos);
	extern int			IsFileEof(FileHandle handle);

	//////////////////////////////////////////////////////////////////////
	//	I/O buffer
	class	IOBuffer
	{
	public:
		enum	ResCode
		{
		RES_EOF   = -2,
		RES_ERROR = -1,
		};
		enum iobuffer_enum
		{
			DEFAULT_IO_BUFFER_SIZE = 4096,
			UNGET_MAX              =    8,
		};
	protected:
		IOBuffer		*next, *prev;
		unsigned		mode;
		unsigned		buffersize;
		unsigned		validsize;
		unsigned		pos;
		BYTE *			buffer;
		sdb::FileHandle	hFile;
		int				unget_level;
		int				unget_ch[UNGET_MAX];
		TCHAR		   *filename;
		unsigned		line, col;

	public:
		IOBuffer();
		virtual ~IOBuffer();
		int			CreateBuffer(unsigned size = DEFAULT_IO_BUFFER_SIZE);
		int			OpenFile(const TCHAR * path, FileOpenMode mode);
		int			CloseFile(void);
		int			ReadBuffer(void);
		BOOL		IsEnd(void);
		int			UnGetChar(int ch);
		int			GetChar(void);
		int			SkipSpace(void);
		int			GetLine(VarBuffer & line);
	};

}; // end of namespace sdb

//////////////////////////////////////////////////////////////////////////
//	SdbLib

class	SdbLib
{
public:
	ScrLib				*scrlib;
	SqlLib::ConnectInfo	conn_info;
	SqlLib				*sqllib;

	sdb::DBInfo			dbinfo;
	sdb::DBDevLib		dbdev;

public:
	SdbLib();
	~SdbLib();

public:
	int			LoadScr(const TCHAR * path);
private:
	int			InitScr(void);
	int			scr_GetDevOpt(void);

public:
	typedef std::list<sdb::DeviceOption *>	DevOptList;
	DevOptList			devoptList;

public:
	SdbDev *		CreateSdbDev(const TCHAR * desc = NULL);

	BOOL		DBIsConnect(void) const { return this->sqllib->IsConnect(); }
	int			DBOpen(void);
	int			DBClose(void);
	int			DBCreate(void);
	int			DBDrop(void);

public:
	typedef std::list<SdbDev *>	Dev_list;
	Dev_list			dev_list;
	void				dev_push_back(SdbDev * dev);
	void				dev_clear(void);
	SdbDev *			dev_search(const TCHAR * desc);

public:
	int					GenerateSmtTblName(SQStr & tblname, sdb::DBDevData & devdata);
	sdb::DeviceOption *	GetDeviceOption(sdb::DBDevData & devdata);

public:	// sdb_file
	const TCHAR *	GetSubPath(SQStr & path);
	const TCHAR *	GetLogText(SQStr & text, sdb::DBDevData & devdata, sdb::DBSmtData & smtdata);
};

//////////////////////////////////////////////////////////////////////////
//	SdbDev - foCX

class	SdbDev
{
public:
	enum	Option
	{
		OPT_GROUPNAME ,
		OPT_HOSTNAME  ,
		OPT_SMART_ATTR,
		OPT_USER_PASS ,
	};

	sdb::DBDevLib		*dbdev;
	sdb::DBSmtLib		*dbsmt;
#if	defined(SDB_USE_DEVCTL)
	DevCtl				*devCtl;
#endif
	SQStr				desc;			// disk<n>
	struct smart_tag
	{
		sdb::SmartAttrList			attrList;
		DevCtl::AtaIdentifyDevice	*identify;
		DevCtl::AtaSmartValues		*val;
		DevCtl::AtaSmartThresholds	*thr;
	} smart;
	GetOpt			*getopt;
	SQStr			model;
	SQStr			firm;
	SQStr			serial;
	SQStr			hostname;
	SQStr			groupname;
	unsigned		blockSize;
	sdb::Int64		blockNumber;
	unsigned		user_id;
	SQStr			user_pass;
	SQStr			memo;

private:

public:
	SdbDev();
	~SdbDev();

#if	defined(WIN32)
	static BOOL	IsHandle(HANDLE handle) { return handle != NULL && handle != INVALID_HANDLE_VALUE; }
#elif	defined(LINUX)
	static BOOL	IsHandle(int handle) { return handle < 0 ? FALSE : TRUE; }
#endif
	static int		GetDeviceName(SQStr & devname, const TCHAR * desc);
#if	defined(SDB_USE_DEVCTL)
	int				OpenDevice(unsigned rwmode);
	int				CloseDevice(void);
	int				GetSmart(void);
	static int		GetThreshold(unsigned id, DevCtl::AtaSmartThresholds *thr);
	int				GetThreshold(unsigned id) { return SdbDev::GetThreshold(id, this->smart.thr); }
	int				GetDeviceData(void);
	static sdb::Uint64	GetAtaCapacity(DevCtl::AtaIdentifyDevice & identify);
	static TCHAR *	FormatAtaString(TCHAR * out, const DevCtl::ATA_WORD * inp, int n);
#endif

	int				RemoveSmartDevice(sdb::DBDevData & devdata);

	int				SmartUpdate(sdb::DBDevData & devdata, sdb::DBSmtData & smtdata);
	int				SmartUpdateFile(sdb::DBDevData & devdata, const TCHAR * path);

	int				ParseOption(const TCHAR * option);
	static TCHAR *	GetSmartFlagsText(TCHAR * buffer, unsigned stt);
	static TCHAR *	GetSmartFlagsText(TCHAR * buffer, BYTE flags[2]) { return SdbDev::GetSmartFlagsText(buffer, sdb::ATA_GET16(flags)); }
};

#endif
