//////////////////////////////////////////////////////////////////////////
//	"scrlib.cpp"
//------------------------------------------------------------------------
//	$Id: scrlib.cpp 71 2006-03-28 03:12:32Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"scrlib.h"
#include	<string.h>
#include	<ctype.h>
#include	<stdio.h>

ScrLib::ScrLib()
{
#if	defined(WIN32)
	this->io.hFile  = INVALID_HANDLE_VALUE;
#else
	this->io.fp     = NULL;
#endif
	this->io.buffer = NULL;
	this->io.unget_flag = FALSE;
}

ScrLib::~ScrLib()
{
#if	defined(WIN32)
	if ( this->io.hFile != INVALID_HANDLE_VALUE )
	{
		::CloseHandle(this->io.hFile);
		this->io.hFile = INVALID_HANDLE_VALUE;
	}
#else
	if ( this->io.fp != NULL )
	{
		fclose(this->io.fp);
		this->io.fp = NULL;
	}
#endif
	if ( this->io.buffer != NULL )
	{
		delete [] this->io.buffer;
		this->io.buffer = NULL;
	}
}

// ݒ
int		ScrLib::Init(void)
{
	static struct tag_inibtl
	{
		TCHAR		*name;
		unsigned	type;
	} tbl[] = 
	{
		{ TEXT("int")   , T_INT    |T_FLAGTYPE },
		{ TEXT("struct"), T_STRUCT             },
		{ TEXT("string"), T_STRING |T_FLAGTYPE },
	};
	int			i;

	for ( i = 0; i < sizeof(tbl)/sizeof(*tbl); i++ )
	{
		ScType	   *scType;

		if ( (scType = new ScType) == NULL )
			return -1;
		scType->name.SetText(tbl[i].name);
		scType->type = tbl[i].type;
		scType->size = 0;
		this->AddType(this->global, scType);
	}
	return 0;
}


int		ScrLib::OpenFile(const TCHAR * path)
{
#if	defined(WIN32)
	if ( this->io.hFile != INVALID_HANDLE_VALUE )
		return -1;
#else
	if ( this->io.fp != NULL )
		return -1;
#endif
	if ( this->io.buffer == NULL )
	{
		if ( this->CreateIoBuf() < 0 )
			return -1;
	}
#if	defined(WIN32)
	this->io.hFile = ::CreateFile(
	  path,
	  GENERIC_READ,
	  FILE_SHARE_READ,
	  NULL,
	  OPEN_EXISTING ,
	  FILE_ATTRIBUTE_NORMAL,
	  NULL);
	if ( this->io.hFile == INVALID_HANDLE_VALUE )
		return -1;
#else
	if ( (this->io.fp = fopen(path, "r")) == NULL )
		return -1;
#endif
	this->io.lno = 0;
	this->io.cno = 0;
	return 0;
}

int		ScrLib::CloseFile(void)
{
#if	defined(WIN32)
	if ( this->io.hFile == INVALID_HANDLE_VALUE )
		return 0;
	::CloseHandle(this->io.hFile);
	this->io.hFile = INVALID_HANDLE_VALUE;
#else
	if ( this->io.fp == NULL )
		return 0;
	fclose(this->io.fp);
	this->io.fp = NULL;
#endif
	return 0;

}

int		ScrLib::CreateIoBuf(unsigned size)
{
#if	defined(WIN32)
	if ( this->io.hFile != INVALID_HANDLE_VALUE )
		return -1;
#else
	if ( this->io.fp != NULL )
		return -1;
#endif
	if ( this->io.buffer != NULL )
		delete [] this->io.buffer;
	if ( (this->io.buffer = new BYTE [size]) == NULL )
		return -1;
	this->io.bufsize   = size;
	this->io.validsize = 0;
	this->io.pos       = 0;
	return 0;
}

int		ScrLib::ReadBuffer(void)
{
#if	defined(WIN32)
	DWORD	size;

	if ( this->io.hFile == INVALID_HANDLE_VALUE || this->io.buffer == NULL )
		return -1;
	if ( ::ReadFile(this->io.hFile, this->io.buffer, this->io.bufsize, &size, NULL) == 0 )
		return -1;
	this->io.validsize = size;
	return (int)size;
#else
	size_t	size;

	if ( this->io.fp == NULL || this->io.buffer == NULL )
		return -1;
	if ( (size = fread(this->io.buffer, 1, this->io.bufsize, this->io.fp)) == 0 )
		return -1;
	this->io.validsize = size;
	return (int)size;
#endif
}

int		ScrLib::UnGetChar(int ch)
{
	this->io.unget_ch   = ch;
	this->io.unget_flag = TRUE;
	return 0;
}

int		ScrLib::GetChar(void)
{
	int		ch = 0, ch2;
	BOOL	loop = FALSE;

	if ( this->io.unget_flag )
	{
		this->io.unget_flag = FALSE;
		return this->io.unget_ch;
	}
	for (;;)
	{
		if ( this->io.pos >= this->io.validsize )
		{
			if ( this->ReadBuffer() < 0 || this->io.validsize == 0 )
				return -1;
			this->io.pos = 0;
		}
		ch = this->io.buffer[this->io.pos++];
		if ( loop == FALSE )
		{
			if ( _ismbblead(ch) == FALSE )
				break;
			ch2 = ch;
			loop = TRUE;
			continue;
		} else
		{
			if ( _ismbbtrail(ch) )
				ch = (ch2 << 8) | ch;
			else
				this->io.pos--;
			break;
		}
	}
	if ( ch == '\n' )
	{
		this->io.lno++;
		this->io.cno = 0;
	}
	return ch;
}

void	ScrLib::SkipSpace(void)
{
	int			ch, ch2;

	for (;;)
	{
		if ( (ch = this->GetChar()) < 0 )
			return;
		switch ( ch )
		{
			case ' ':
			case '\t':
			case '\r':
			case '\n':
				break;
			case '/':
				if ( (ch2 = this->GetChar()) < 0 )
				{
					this->UnGetChar(ch);
					return;
				}
				if ( ch == '/' || ch == '*' )
				{
					this->SkipComment(ch);
				} else
				{
					this->UnGetChar(ch2);
					return;
				}
				break;
			default:
				this->UnGetChar(ch);
				return;
		}
	}
}

void	ScrLib::SkipComment(int ch)
{
	int		ch2;

	if ( ch == '*' )
	{	//
		int		level = 1;

		while ( (ch = this->GetChar()) >= 0 )
		{	/* */
			if ( (ch2 = this->GetChar()) >= 0 )
				this->UnGetChar(ch2);
			if ( ch == '*' && ch2 == '/' )
			{
				this->GetChar();
				level--;
				if ( level == 0 )
					break;
			} else if ( ch == '/' && ch2 == '*' )
			{
				this->GetChar();
				level++;
			}
		}
	} else
	{	// s܂ŃRg
		while ( (ch = this->GetChar()) >= 0 )
		{
			if ( ch == '\n' )
				break;
		}
	}
}

int		ScrLib::GetCharCheck(int check)
{
	int		ch;

	this->SkipSpace();
	if ( (ch = this->GetChar()) < 0 )
		return -1;
	if ( ch == check )
		return 1;
	this->UnGetChar(ch);
	return 0;
}

int		ScrLib::GetEscChar(void)
{
	int			ch;

	if ( (ch = this->GetChar()) < 0 )
		return -1;
	switch ( ch )
	{
		case 'n': ch = '\n'; break;
		case 'r': ch = '\r'; break;
		case 't': ch = '\t'; break;
		case '0': ch = '\0'; break;
		case '\r':
			if ( (ch = this->GetChar()) < 0 )
				return -1;
			if ( ch != '\n' )
			{
				this->UnGetChar(ch);
				return '\r';
			}
		case '\n':		//	̍sɑ
			return -1;
	}
	return ch;
}

int		ScrLib::GetWord(SQStr & word)
{
	int			ch;
	BYTE		buffer[4096];
	unsigned	size = 0;

	this->SkipSpace();
	if ( (ch = this->GetChar()) < 0 )
		return -1;
	if ( this->IsNameTopChar(ch) == FALSE )
		return -1;
	for(;;)
	{
		if ( size >= sizeof(buffer) - 1 )
			break;
		if ( ch >= 256 )
		{
			buffer[size++] = ch >> 8;
			if ( size >= sizeof(buffer) - 1 )
				break;
		}
		buffer[size++] = ch & 0xFF;
		if ( (ch = this->GetChar()) < 0 )
			break;
		if ( this->IsNameChar(ch) == FALSE )
		{
			this->UnGetChar(ch);
			break;
		}
	}
	buffer[size] = 0;
	word.SetText((TCHAR *)buffer);

	return 0;
}

int		ScrLib::IsNameTopChar(int ch)
{
	if ( (ch >= 'A' && ch <= 'Z')
	  || (ch >= 'a' && ch <= 'z')
	  || ch == '_' )
		return 1;
	return 0;
}

int		ScrLib::IsNameChar(int ch)
{
	if ( (ch >= 'A' && ch <= 'Z')
	  || (ch >= 'a' && ch <= 'z')
	  || (ch >= '0' && ch <= '9')
	  || ch == '_' )
		return 1;
	return 0;
}

// ^ǉ
int		ScrLib::AddType(NameData & ns, ScrLib::ScType * scType)
{
	if ( ns.GetItem(scType->name) != NULL )
		return -1;
	ns.SetItem(scType);
	return 0;
}

ScrLib::ScValue *	ScrLib::GetValue(SQStr & name)
{
	ScValue	   *scValue;

	if ( (scValue = (ScValue *)this->global.GetItem(name)) == NULL )
		return NULL;
	if ( (scValue->type & T_FLAGVALUE) == 0 )
		return NULL;
	return scValue;
}

ScrLib::ScType *	ScrLib::EnumType(ScrLib::ScType * scType)
{
	return (ScType *)this->global.EnumItem(scType);
}

//////////////////////////////////////////////////////////////////////////
//	Parse Main

int		ScrLib::Parse(void)
{
	int			ret;

	for (;;)
	{
		int			ch;

		ret = 0;
		this->SkipSpace();
		if ( (ch = this->GetChar()) < 0 )
			break;	// normal end
		if ( this->IsNameTopChar(ch) )
		{
			SQStr			word;
			ScType		   *scType;

			this->UnGetChar(ch);
			if ( this->GetWord(word) < 0 )
			{
				ret = -1;
				break;
			}
			if ( (scType = (ScType *)this->global.GetItem(word)) != NULL )
			{	// o^ς݂̖O
				if ( (scType->type & T_FLAGTYPE) != 0 )
				{	// ^
					ret = this->parse_type(*scType);
				} else
				{	// ϐ
					switch ( scType->type & T_NAMEMASK )
					{
						case T_STRUCT:		// \̒`
							ret = this->parse_struct();
							break;
						default:			// error ex.
							ret = -1;
							break;
					}
				}
			} else
			{	// error: o^̖O
				ret = -1;
				break;
			}
		} else
		{	// error: łȂ
			ret = -1;
			break;
		}
		if ( ret == 0 )
		{
			if ( this->GetCharCheck(';') < 0 )
				ret = -1;
		}
		if ( ret < 0 )
			break;
	}
	if ( ret < 0 )
	{
		TCHAR	msg[1024];

		::_stprintf(msg, TEXT("`t@C %d s (%d )ŃG[܂B"), this->io.lno, this->io.cno);
#if	defined(SDBCTL) || defined(SDBWATCH)
		_fputts(msg, stderr);
		_fputts(TEXT("\n"), stderr);
#elif	defined(WIN32)
		::MessageBox(NULL, msg, "`t@C̓ǂݍ݃G[", MB_OK|MB_ICONWARNING);
#endif
	}
	return ret;
}

// \̒`: ex. struct <*>YbbInfo  { .... };
int		ScrLib::parse_struct(void)
{
	SQStr		name;
	ScType	   *scTypeStruct;
	int			ch;

	if ( this->GetWord(name) < 0 )
		return -1;	// error
	if ( this->global.GetItem(name) != NULL )
		return -1;	// ɓo^Ă
	if ( (scTypeStruct = new ScType) == NULL )
		return -1;
	scTypeStruct->name = name;
	scTypeStruct->type  = T_STRUCT | T_FLAGTYPE;
	scTypeStruct->size  = 0;
	scTypeStruct->def.memb = new NameData;

	if ( this->GetCharCheck('{') <= 0 )
		return -1;
	for (;;)
	{
		this->SkipSpace();
		if ( (ch = this->GetChar()) < 0 )
			return -1;
		if ( this->IsNameTopChar(ch) )
		{
			SQStr				word;
			NameItem		   *niname;
			ScType			   *scType;

			// ^
			this->UnGetChar(ch);
			if ( this->GetWord(word) < 0 )
				return -1;
			if ( (scType = (ScType *)this->global.GetItem(word)) == NULL )
				return -1;
			if ( (scType->type & T_FLAGTYPE) == 0 )
				return -1;

			// oϐ
			if ( this->GetWord(word) < 0 )
				return -1;
			if ( (niname = this->global.GetItem(word)) != NULL )
				return -1;
			if ( (niname = scTypeStruct->def.memb->GetItem(word)) != NULL )
				return -1;

			// oǉ
			scTypeStruct->AddMember(word, scType);

			if ( this->GetCharCheck(';') <= 0 )
				return -1;
		} else
		{
			if ( ch != '}' )
				return -1;
			break;
		}
	}
//	if ( this->GetCharCheck(';') <= 0 )
//		return -1;

	this->AddType(this->global, scTypeStruct);

	return 0;
}

// ϐ`: ex. YbbInfo <*>tbl[] = { ... };
//			ex.	int	value = 1234;
int		ScrLib::parse_type(ScrLib::ScType & scType)
{
	SQStr		name;
	BOOL		array = FALSE;
	int			ch;

	// ϐ
	if ( this->GetWord(name) < 0 )
		return -1;	// error

	if ( this->global.GetItem(name) != NULL )
		return -1;	// w̕ϐ͊ɎgĂ

	this->SkipSpace();
	if ( (ch = this->GetChar()) < 0 )
		return -1;
	if ( ch == '[' )
	{	// z
		if ( this->GetCharCheck(']') <= 0 )
			return -1;
		array = TRUE;
	} else
		this->UnGetChar(ch);
	if ( this->GetCharCheck('=') <= 0 )
		return -1;

	{
		ScValue	   *scValue;

		if ( array == FALSE )
			scValue = scType.ParseCreateValue(*this, name);
		else
			scValue = scType.ParseCreateValueArray(*this, name);
		if ( scValue == NULL )
			return -1;
		this->global.SetItem(scValue);
	}
//	if ( this->GetCharCheck(';') <= 0 )
//		return -1;
	return 0;
}

int		ScrLib::parse_value(ScrLib::ScValue & scValue)
{
	int			ch;
	unsigned	type_flags = (scValue.type & ~T_NAMEMASK);
	BOOL		minus = FALSE;

	this->SkipSpace();
	if ( (ch = this->GetChar()) < 0 )
		return -1;
	if ( ch == '-' )
	{
		minus = TRUE;
		if ( (ch = this->GetChar()) < 0 )
			return -1;
	}

	if ( ch == '0' )
	{
		if ( (ch = this->GetChar()) < 0 )
			return -1;
		if ( ch == 'x' || ch == 'X' )	// 16i
		{
			unsigned	u;

			u = 0;
			while ( (ch = this->GetChar()) >= 0 )
			{
				if ( ch >= '0' && ch <= '9' )
					u = (u<<4) | (ch - '0');
				else if ( ch >= 'A' && ch <= 'F' )
					u = (u<<4) | (ch - 'A' + 10);
				else if ( ch >= 'a' && ch <= 'f' )
					u = (u<<4) | (ch - 'a' + 10);
				else
				{
					this->UnGetChar(ch);
					break;
				}
			}
			scValue.Destroy();
			scValue.type = T_INT | type_flags;
			scValue.val.si = minus ? -(int)u : (int)u;
		} else
			goto NUM;
	} else if ( ch >= '0' && ch <= '9' )
	{
		unsigned		u;

NUM:
		this->UnGetChar(ch);
		u = 0;
		while ( (ch = this->GetChar()) >= 0 )
		{
			if ( ch >= '0' && ch <= '9' )
				u = u * 10 + (ch - '0');
			else
			{
				this->UnGetChar(ch);
				break;
			}
		}
		scValue.Destroy();
		scValue.type = T_INT | type_flags;
		scValue.val.si = minus ? -(int)u : (int)u;
	} else if ( ch == '\'' )	// 
	{
		if ( (ch = this->GetChar()) < 0 )
			return -1;
		if ( ch == '\\' )
			ch = this->GetEscChar();
		scValue.Destroy();
		scValue.type = T_INT | type_flags;
		scValue.val.si = minus ? -ch : ch;
		ch = this->GetChar();
		if ( ch != '\'' )
			return -1;
	} else if ( ch == '\"' )	// 
	{
		BYTE	buffer[4096];
		int		pos = 0;

		if ( minus )
			return -1;
		while ( (ch = this->GetChar()) >= 0 && pos < sizeof(buffer) - 1 )
		{
			if ( ch == '\"' )
				break;
			if ( ch == '\\' )
			{
				if ( (ch = this->GetEscChar()) < 0 )
					continue;
			}
			if ( ch >= 256 )	// QoCg
				buffer[pos++] = ch >> 8;
			buffer[pos++] = ch;
		}
		buffer[pos++] = 0;
		scValue.Destroy();
		scValue.type = T_STRING | type_flags;
		scValue.val.str = new SQStr;
		scValue.val.str->SetText((TCHAR *)buffer);
	} else
		return -1;

	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	ScType

ScrLib::ScType::ScType()
{
	this->type = 0;
	this->size = 0;
	this->def.memb = NULL;
}

ScrLib::ScType::~ScType()
{
	switch ( this->type & T_NAMEMASK )
	{
		case T_STRUCT:
			if ( this->def.memb != NULL )
			{
				delete this->def.memb;
				this->def.memb = NULL;
			}
			break;
	}
}

ScrLib::ScValue *		ScrLib::ScType::CreateValue(SQStr & name)
{
	ScValue	   *scValue;

	if ( (scValue = new ScValue) == NULL )
		return NULL;
	scValue->type_name = this->name;
	scValue->name.SetText(name);
	scValue->type = (this->type & T_NAMEMASK) | T_FLAGVALUE;
//	switch ( scValue->type & T_NAMEMASK )
//	{
//		case T_STRUCT:
//			this->def.memb = new NameData;
//			break;
//	}
	return scValue;
}

// T_FLAGARRAY ͖
ScrLib::ScValue *		ScrLib::ScType::ParseCreateValue(ScrLib & scrlib, SQStr & name)
{
	ScValue	   *scValue = NULL;

	switch ( this->type & T_NAMEMASK )
	{
		case T_STRUCT:
			scValue = this->ParseCreateValueStruct(scrlib, name);
			break;
		case T_INT:
			scValue = this->ParseCreateValueInt(scrlib, name);
			break;
		case T_STRING:
			scValue = this->ParseCreateValueString(scrlib, name);
			break;
	}
	return scValue;
}

// z tbl[] = <*>{ ... };
ScrLib::ScValue *		ScrLib::ScType::ParseCreateValueArray(ScrLib & scrlib, SQStr & name)
{
	ScValue		   *scValueArray;
	int				ch, n;
	BOOL			flagCom = FALSE;
	if ( (scValueArray = this->CreateValue(name)) == NULL )
		return NULL;
	scValueArray->type |= T_FLAGARRAY;
	if ( (scValueArray->val.a = new NameData) == NULL )
		return NULL;

	scrlib.SkipSpace();
	if ( (ch = scrlib.GetChar()) < 0 )
		return NULL;
	if ( ch != '{' )
		return NULL;

	// zvf̏
	n = 0;
	for (;;)
	{
		TCHAR		sn[80];
		SQStr		number;
		ScValue	   *scValue;

		scrlib.SkipSpace();
		if ( (ch = scrlib.GetChar()) < 0 )
			return NULL;
		if ( ch == '}' )
			break;
		if ( ch == ',' )
		{
			flagCom = TRUE;
			scrlib.SkipSpace();
			if ( (ch = scrlib.GetChar()) < 0 )
				return NULL;
			if ( ch == '}' )	// ex. ",}"
				break;
			scrlib.UnGetChar(ch);
		} else
		{
			flagCom = FALSE;
			scrlib.UnGetChar(ch);
		}

		_stprintf(sn, "%d", n);
		number.SetText(sn);
		if ( (scValue = this->ParseCreateValue(scrlib, number)) == NULL )
			return NULL;
		scValueArray->val.a->SetItem(scValue);
		n++;
	}

	return scValueArray;
}


// \ YbbInfo tbl = <*>{ ... };
ScrLib::ScValue *		ScrLib::ScType::ParseCreateValueStruct(ScrLib & scrlib, SQStr & name)
{
	ScValue		   *scValueStruct;
	ScType		   *scTypeMemb = NULL;
	BOOL			memb_end = FALSE;
	int				ch;

	if ( (this->type & T_NAMEMASK ) != T_STRUCT || this->def.memb == NULL )	// \̂ł͂Ȃ
		return NULL;

	if ( (scValueStruct = this->CreateValue(name)) == NULL )
		return NULL;
	if ( scValueStruct->val.memb == NULL )
	{
		if ( (scValueStruct->val.memb = new NameData) == NULL )
			return NULL;
	}
	scrlib.SkipSpace();
	if ( (ch = scrlib.GetChar()) < 0 )
		return NULL;
	if ( ch != '{' )
		return NULL;
	while ( (scTypeMemb = (ScType *)this->def.memb->EnumItem(scTypeMemb)) != NULL )
	{
		ScValue	   *scValueMemb, *scValueMemb2;

		if ( (scValueMemb = scTypeMemb->ParseCreateValue(scrlib, scTypeMemb->name)) == NULL )
			return NULL;
		scValueMemb2 = (ScValue *)scValueStruct->val.memb->GetItem(scValueMemb->name);
		if ( scValueMemb2 != NULL )
		{	// oϐo^ĂBo[Wł̓G[ƂB
			return NULL;
		} else
		{	// oϐeݒ
			scValueStruct->val.memb->SetItem(scValueMemb);
		}

		scrlib.SkipSpace();
		if ( (ch = scrlib.GetChar()) < 0 )
			return NULL;
		if ( ch == ',' )
			continue;
		if ( ch == '}' )
		{
			memb_end = TRUE;
			break;
		} else
			return NULL;
	}
	if ( memb_end == FALSE )
		return NULL;
	return scValueStruct;
}

ScrLib::ScValue *		ScrLib::ScType::ParseCreateValueInt(ScrLib & scrlib, SQStr & name)
{
	ScValue		   *scValue;

	if ( (this->type & T_NAMEMASK ) != T_INT )
		return NULL;
	if ( (scValue = this->CreateValue(name)) == NULL )
		return NULL;
	scrlib.SkipSpace();
	if ( scrlib.parse_value(*scValue) < 0 )
		return NULL;
	if ( (scValue->type & T_NAMEMASK) != T_INT )
		return NULL;
	return scValue;
}

ScrLib::ScValue *		ScrLib::ScType::ParseCreateValueString(ScrLib & scrlib, SQStr & name)
{
	ScValue		   *scValue;

	if ( (this->type & T_NAMEMASK ) != T_STRING )
		return NULL;
	if ( (scValue = this->CreateValue(name)) == NULL )
		return NULL;
	scrlib.SkipSpace();
	if ( scrlib.parse_value(*scValue) < 0 )
		return NULL;
	if ( (scValue->type & T_NAMEMASK) != T_STRING )
		return NULL;
	return scValue;
}

// o`ǉ
int		ScrLib::ScType::AddMember(SQStr & name, ScrLib::ScType * scType)
{
	ScType		   *scTypeMemb;

	if ( (this->type & T_NAMEMASK ) != T_STRUCT || this->def.memb == NULL )	// \̂ł͂Ȃ
		return -1;
	if ( this->def.memb->GetItem(name) != NULL )
		return -1;
	if ( (scTypeMemb = new ScType) == NULL )
		return -1;
	scTypeMemb->name.SetText(name);
	scTypeMemb->type = scType->type;
	scTypeMemb->size = scType->size;
	this->def.memb->SetItem(scTypeMemb);
	this->size += scTypeMemb->size;
	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	ScValue

ScrLib::ScValue::ScValue()
{
	this->val.data = NULL;
}

ScrLib::ScValue::~ScValue()
{
	this->Destroy();
}

void	ScrLib::ScValue::Destroy(void)
{
	if ( this->type & T_FLAGARRAY )
	{	// z
		if ( this->val.a != NULL )
		{
			delete this->val.a;
			this->val.a = NULL;
		}
	} else
	{
		switch ( this->type & T_NAMEMASK )
		{
			case T_STRING:
				if ( this->val.str != NULL )
				{
					delete this->val.str;
					this->val.str = NULL;
				}
				break;

			case T_STRUCT:
				if ( this->val.memb != NULL )
				{
					delete this->val.memb;
					this->val.memb = NULL;
				}
				break;
		}
	}
	this->type = 0;
}

ScrLib::ScValue *	ScrLib::ScValue::CreateValue(SQStr & name, ScrLib::ScType & scType) // static
{
	return scType.CreateValue(name);
}

ScrLib::ScValue *	ScrLib::ScValue::GetArrayValue(int n)
{
	ScValue	   *scValue;
	TCHAR		sn[80];
	SQStr		number;

	if ( (this->type & T_FLAGARRAY) == 0 || this->val.a == NULL )
		return NULL;
	_stprintf(sn, "%d", n);
	number.SetText(sn);
	scValue = (ScValue *)this->val.a->GetItem(number);
	if ( scValue == NULL || (scValue->type & T_FLAGVALUE) == 0 )
		return NULL;
	return scValue;
}

ScrLib::ScValue *	ScrLib::ScValue::GetMemb(SQStr & name)
{
	if ( (this->type & T_NAMEMASK) != T_STRUCT || this->val.memb == NULL )
		return NULL;
	return (ScValue *)this->val.memb->GetItem(name);
}

ScrLib::ScValue *	ScrLib::ScValue::EnumMemb(ScrLib::ScValue * scValue)
{
	if ( (this->type & T_NAMEMASK) != T_STRUCT || this->val.memb == NULL )
		return NULL;
	return (ScValue *)this->val.memb->EnumItem(scValue);
}


