//////////////////////////////////////////////////////////////////////////
//	"socklib.cpp"
//------------------------------------------------------------------------
//	$Id: socklib.cpp 66 2006-03-27 20:14:43Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"socklib.h"
#include	<stdlib.h>
#include	<ctype.h>
#include	<string.h>
#if	defined(WIN32)
# include	<tchar.h>
#endif

#if defined(WIN32)
# if	SOCKLIB_WINSOCK_VER == 1
#pragma comment(lib, "wsock32.lib")
# endif
# if	SOCKLIB_WINSOCK_VER == 2
#pragma	comment(lib, "ws2_32.lib")
# endif
#endif

//////////////////////////////////////////////////////////////////////////
//	SockAddr
//////////////////////////////////////////////////////////////////////////

SockLib::SockAddr::SockAddr()
{
	::memset(&this->sockaddr, 0, sizeof(this->sockaddr));
	this->sockaddr.sin_family = AF_INET;
}

int		SockLib::SockAddr::SetAddr(unsigned long addr, unsigned port)
{
	memset(&this->sockaddr, 0, sizeof(this->sockaddr));
	this->sockaddr.sin_family = AF_INET;
	this->SetAddr(addr);
	this->SetPort(port);
	return 0;
}

int		SockLib::SockAddr::SetAddr(const char * s)
{
	int		ret;

	if ( SockLib::sockInit == FALSE )
		SockLib::Init();
	if ( isalpha(s[0]) )
	{	// name
		struct hostent	*hp;

		hp = ::gethostbyname(s);
		if ( hp == NULL )
		{
			ret = -1;
			goto EXIT;
		}
		::memset(&this->sockaddr, 0, sizeof(this->sockaddr));
		this->sockaddr.sin_family = hp->h_addrtype;
		memcpy(&this->sockaddr.sin_addr, hp->h_addr, hp->h_length);
	} else
	{	// xxx.xxx.xxx.xxx
		unsigned long	a;

		if ( (a = ::inet_addr(s)) == INADDR_NONE )
		{
			ret = -1;
			goto EXIT;
		}
		::memset(&this->sockaddr, 0, sizeof(this->sockaddr));
		this->sockaddr.sin_family = AF_INET;
		this->sockaddr.sin_addr.s_addr = a;
	//	memset(this->sockaddr.sin_zero, 0, sizeof(this->sockaddr.sin_zero));
	//	hp = ::gethostbyaddr((char *)&a, 4, AF_INET);
	//	if ( hp == NULL )
	//		return 0;
	}
	ret = 0;
EXIT:
	if ( SockLib::sockCount == 0 && SockLib::sockInit != FALSE )
		SockLib::Term();
	return ret;
}

const char	*SockLib::SockAddr::GetAddrText(void)
{
	return ::inet_ntoa(this->sockaddr.sin_addr);
}

const char	*SockLib::SockAddr::GetAddrText(unsigned length, char * buffer)
{
	const char	*text;

	if ( length == 0 )
		return NULL;
	if ( (text = ::inet_ntoa(this->sockaddr.sin_addr)) == NULL )
		return NULL;
	strncpy(buffer, text, length);
	return buffer;
}

struct hostent	*SockLib::SockAddr::GetHostent(void)
{
	return ::gethostbyaddr((char *)&this->sockaddr.sin_addr.s_addr, sizeof(this->sockaddr.sin_addr.s_addr), AF_INET);
}

//////////////////////////////////////////////////////////////////////////
//	SockLib
//////////////////////////////////////////////////////////////////////////

int			SockLib::sockInit   = FALSE;
int			SockLib::sockCount  = 0;
#if	defined(WIN32)
# if	SOCKLIB_WINSOCK_VER == 1
WORD		SockLib::sockVer    = MAKEWORD(1, 1);
# endif
# if	SOCKLIB_WINSOCK_VER == 2
WORD		SockLib::sockVer    = MAKEWORD(2, 0);
# endif
WSADATA		SockLib::wsaData;
#endif

SockLib::SockLib()
{
	this->flags = 0;
	this->socket_type = SOCK_STREAM;	// tcp
	this->socket_proto = IPPROTO_IP;
	this->conn_socket = 0;
}

SockLib::~SockLib()
{
	if ( this->flags )
		this->Close();
}

int		SockLib::Init(void) // static
{
	if ( SockLib::sockInit == FALSE )
	{
#if defined(WIN32)
		static WORD	wsver[] = 
		{
#if	SOCKLIB_WINSOCK_VER == 2
			MAKEWORD(2,0),
			MAKEWORD(1,1),
			MAKEWORD(1,0),
#endif
#if	SOCKLIB_WINSOCK_VER == 1
			MAKEWORD(1,1),
			MAKEWORD(1,0),
#endif
		};
		int				i, success;

		success = 0;
		for ( i = 0; i < sizeof(wsver)/sizeof(*wsver); i++ )
		{
			if ( ::WSAStartup(wsver[i], &SockLib::wsaData) == 0 )
			{
				success = 1;
				SockLib::sockVer = wsver[i];
				break;
			} else
			{
				::WSACleanup();
			}
		}
		if ( success == 0 )
			return -1;	// error
#endif
		SockLib::sockInit = TRUE;
	}

	return 0;
}

void	SockLib::Term(void)
{
	if ( SockLib::sockCount == 0 && SockLib::sockInit != FALSE )
	{
#if defined(WIN32)
		::WSACleanup();
#endif
		SockLib::sockInit = FALSE;
	}
	return;
}

//#define	SOCKLIB_USE_SELECT_CONNECT

int		SockLib::Create(int type, int proto)
{
	if ( this->flags & F_SOCKET )
		return -1;

	if ( SockLib::sockInit == FALSE )
		SockLib::Init();

	this->socket_type = type;
	this->socket_proto = proto;
	this->conn_socket = ::socket(AF_INET, this->socket_type, this->socket_proto);
	if ( this->conn_socket < 0 )
	{
		this->Close();
		return -1;	// error
	}
#if	defined(SOCKLIB_USE_SELECT_CONNECT)
	{
		 u_long	arg;

		arg = 1;
		ioctlsocket(this->conn_socket, FIONBIO, &arg);
	}
#endif
	this->flags |= F_SOCKET;
	SockLib::sockCount++;
	return 0;
}

int		SockLib::Close(void)
{
	if ( this->flags & F_SOCKET )
	{
#if	defined(WIN32)
		::closesocket(this->conn_socket);
#else
		close(this->conn_socket);
#endif
		this->flags &= ~F_SOCKET;
		SockLib::sockCount--;
	}
	if ( SockLib::sockCount == 0 && SockLib::sockInit != FALSE )
		SockLib::Term();
	return 0;
}

int		SockLib::SetSocket(SOCKET socket)
{
	if ( this->flags & F_SOCKET )
		return -1;
	this->conn_socket = socket;
	this->flags |= F_SOCKET;
	SockLib::sockCount++;
	return 0;
}

int		SockLib::GetSockOpt(int level, int optname, void * optval, socklen_t * optlen)
{
	int			ret;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::getsockopt(this->conn_socket, level, optname, (char *)optval, optlen)) == SOCKET_ERROR )
		return -1;
	return ret;
}

int		SockLib::SetSockOpt(int level, int optname, void * optval, socklen_t optlen)
{
	int			ret;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::setsockopt(this->conn_socket, level, optname, (char *)optval, optlen)) != 0 )
		return ret;
	return ret;
}

#include	<stdio.h>

int		SockLib::Connect(SockLib::SockAddr & addr)
{
	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
#if	defined(SOCKLIB_USE_SELECT_CONNECT)
	printf("connect()\n");
	if ( ::connect(this->conn_socket, (struct sockaddr *)&addr.sockaddr, sizeof(addr.sockaddr)) == SOCKET_ERROR )
	{
		int					ret, err;
		struct timeval		tm;
		SOCKET				width;
		fd_set				fds;
		DWORD				tick_start;

		err = WSAGetLastError();
		printf("connect error (%d)\n", err);
		if ( err != WSAEWOULDBLOCK )
		{
			printf("non block error\n");
			return -1;	// error
		}
		tm.tv_sec  = 1000 / 1000;
		tm.tv_usec = 1000 % 1000;
		width = this->conn_socket + 1;
		FD_ZERO(&fds);
		FD_SET(this->conn_socket, &fds);
		tick_start = GetTickCount();
		while ( GetTickCount() - tick_start < 30 * 1000 )
		{
			printf("select()\n");
			if ( ::select(width, NULL, &fds, NULL, &tm) < 0 )
			{	// ڑ҂
				err = ::WSAGetLastError();
				printf("select error (%d)\n", err);
			} else
			{
				break;	// OK
			}
		}
		if ( FD_ISSET(this->conn_socket, &fds) )
		{
			printf("OK\n");
			ret = 0;	// success
		} else
		{
			err = ::WSAGetLastError();
			printf("connect error (%d)\n", err);
			return -1;
		}
	}
#else
	if ( ::connect(this->conn_socket, (struct sockaddr *)&addr.sockaddr, sizeof(addr.sockaddr)) == SOCKET_ERROR )
		return -1;
#endif

	return 0;
}

int		SockLib::Bind(SockLib::SockAddr & addr)
{
	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( ::bind(this->conn_socket, (struct sockaddr *)&addr.sockaddr, sizeof(addr.sockaddr)) == SOCKET_ERROR )
		return -1;
	return 0;
}

int		SockLib::Listen(int backlog)
{
	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( ::listen(this->conn_socket, backlog) == SOCKET_ERROR )
		return -1;
	return 0;
}

int		SockLib::Accept(SockLib * socklib, SockLib::SockAddr & addr)
{
	socklen_t	addrlen = sizeof(addr.sockaddr);
	SOCKET		socket;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (socket = ::accept(this->conn_socket, (struct sockaddr *)&addr.sockaddr, &addrlen)) == SOCKET_ERROR )
		return -1;
	if ( socklib != NULL )
		socklib->SetSocket(socket);
	return 0;
}

int		SockLib::Send(unsigned size, const void * buffer, int flag)
{
	int		ret;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::send(this->conn_socket, (const char *)buffer, size, flag)) == SOCKET_ERROR )
		return -1;
	return ret;
}

int		SockLib::SendTo(unsigned size, const void * buffer, int flag, SockLib::SockAddr & toaddr)
{
	int		ret;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::sendto(this->conn_socket, (const char *)buffer, size, flag, (struct sockaddr *)&toaddr.sockaddr, sizeof(toaddr.sockaddr))) == SOCKET_ERROR )
		return -1;
	return ret;
}

int		SockLib::Recv(unsigned size, void * buffer, int flag)
{
	int		ret;

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::recv(this->conn_socket, (char *)buffer, size, flag)) == SOCKET_ERROR )
		return -1;
	return ret;
}

int		SockLib::RecvFrom(unsigned size, void * buffer, int flag, SockLib::SockAddr & from)
{
	int			ret;
	socklen_t	fromlen = sizeof(from.sockaddr);

	if ( (this->flags & F_SOCKET) == 0 )
		return -1;
	if ( (ret = ::recvfrom(this->conn_socket, (char *)buffer, size, flag, (struct sockaddr *)&from.sockaddr, &fromlen)) == SOCKET_ERROR )
		return -1;
	return ret;
}

//////////////////////////////////////////////////////////////////////////
//	Library

int		SockLib::GetHostName(unsigned size, char * buffer)	// static
{
	int			ret;

	if ( SockLib::sockInit == FALSE )
		SockLib::Init();

	ret = ::gethostname(buffer, size);
	if ( ret == SOCKET_ERROR )	// no error == 0 
		ret = -1;

	if ( SockLib::sockCount == 0 && SockLib::sockInit != FALSE )
		SockLib::Term();
	return ret;
}

