//////////////////////////////////////////////////////////////////////////
//	"sqllib.cpp"
//------------------------------------------------------------------------
//	$Id: sql_pglib.cpp 87 2006-03-30 09:24:05Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"sqllib.h"

#if	defined(SQLLIB_SUPPORT_PGLIB)
#include	<pglib.h>
#include	<stdlib.h>
#include	<string.h>
#if	defined(WIN32)
# include	<tchar.h>
#endif

#if 1 && defined(_DEBUG)
# if	defined(WIN32)
#  include	<crtdbg.h>
#  define	DBGREPORT(param_)	_CrtDbgReport param_
# endif
#else
# define	DBGREPORT(param_)
#endif

int		SqlLib::pglib_init(void)
{
	static FuncTable	func_tbl =
	{
		&SqlLib::pglib_IsConnect   ,
		&SqlLib::pglib_Connect     ,
		&SqlLib::pglib_Finish      ,

		&SqlLib::pglib_Exec        ,
	//	&SqlLib::pglib_ExecRes     ,
		&SqlLib::pglib_ClearResult ,

		&SqlLib::pglib_GetErrorMessage,

		&SqlLib::pglib_GetTypeName ,
		&SqlLib::pglib_SetSqlValue ,

		&SqlLib::pglib_GetStatus   ,
		&SqlLib::pglib_GetTuples   ,
		&SqlLib::pglib_GetFields   ,
		&SqlLib::pglib_GetFname    ,
		&SqlLib::pglib_GetFnumber  ,
		&SqlLib::pglib_GetFtype    ,
		&SqlLib::pglib_GetFmod     ,
		&SqlLib::pglib_GetFsize    ,
		&SqlLib::pglib_GetValue    ,
		&SqlLib::pglib_GetDataValue,
		&SqlLib::pglib_IsNull      ,
		&SqlLib::pglib_GetLength   ,
		&SqlLib::pglib_GetCmdStatus,
		&SqlLib::pglib_GetOidValue ,
	};

	DBGREPORT((_CRT_WARN,NULL,NULL,NULL, "SqlLib::pglib_init\n"));
	if ( this->db.pglib != NULL )
		return -1;
	if ( (this->db.pglib = new PgLib) == NULL )
		return -1;

	this->ftbl = func_tbl;

	return 0;
}

int		SqlLib::pglib_term(void)
{
	DBGREPORT((_CRT_WARN,NULL,NULL,NULL, "SqlLib::pglib_term\n"));
	if ( this->db.pglib != NULL )
	{
		delete this->db.pglib;
		this->db.pglib = NULL;
	}
	return 0;
}

BOOL	SqlLib::pglib_IsConnect(void)
{
	PgLib::ConnStatus	stt;

	if ( this->dbtype != DBTYPE_PGLIB )
		return FALSE;

	if ( this->db.pglib == NULL )
		return FALSE;
	stt = this->db.pglib->GetStatus();
	return stt == PgLib::CONNECTION_OK ? TRUE : FALSE;
}

int		SqlLib::pglib_Connect(const ConnectInfo * info)
{
	VarBuffer			vbuf;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;

	{
		const TCHAR *	host;

		vbuf.AddText(TEXT("host="));
		if ( (host = info->host) == NULL )
			host = TEXT("localhost");
		vbuf.AddText(host);
	}

	if ( info->port != NULL )
	{
		vbuf.AddText(TEXT(" port="));
		vbuf.AddText(info->port);
	}

	vbuf.AddText(TEXT(" dbname="));
	if ( info->dbname == NULL )
		return -1;
	vbuf.AddText(info->dbname);

	if ( info->user != NULL )
	{
		vbuf.AddText(TEXT(" user="));
		vbuf.AddText(info->user);
	}

	if ( info->pass != NULL )
	{
		vbuf.AddText(TEXT(" password="));
		vbuf.AddText(info->pass);
	}

	if ( info->option != NULL )
	{
		vbuf.AddText(TEXT(" "));
		vbuf.AddText(info->option);
	}

	vbuf.AddNulChar();

	DBGREPORT((_CRT_WARN,NULL,NULL,NULL, "SqlLib::pglib_Connect: %s\n", vbuf.GetText()));
	if ( this->db.pglib->Connectdb(vbuf.GetText()) < 0 || this->db. pglib->GetStatus() != PgLib::CONNECTION_OK )
		return -1;
	this->db.pglib->SetClientEncoding("SJIS");
	return 0;
}

int		SqlLib::pglib_Finish(void)
{
	DBGREPORT((_CRT_WARN,NULL,NULL,NULL, "SqlLib::pglib_Finish\n"));
	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	return this->db.pglib->Finish();
}

int		SqlLib::pglib_Exec(SqlLib::Result & result, const TCHAR * query)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( result.create(this->dbtype) < 0 )
		return -1;
	if ( (pg_res = (PgLib::Result *)result.pglib_getres()) == NULL )
		return -1;
	DBGREPORT((_CRT_WARN,NULL,NULL,NULL, "SqlLib::pglib_Exec: %s\n", query));
	return this->db.pglib->Exec(*pg_res, query);
}

#if 0
SqlLib::Result *	SqlLib::pglib_ExecRes(const TCHAR * query)
{
	SqlLib::Result *	result;
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return NULL;
	if ( (result = new Result) == NULL )
		return NULL;
	if ( result->create(this->dbtype) < 0 )
		return NULL;
	if ( (pg_res = (PgLib::Result *)result->pglib_getres()) == NULL )
		return NULL;

	if ( this->db.pglib->Exec(*pg_res, query) < 0 )
	{
		delete result;
		return NULL;
	}
	return result;
}
#endif

void		SqlLib::pglib_ClearResult(SqlLib::Result & result)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return;
	if ( (pg_res = (PgLib::Result *)result.pglib_getres()) != NULL )
		pg_res->Clear();
	result.destroy();

	return;
}

TCHAR *		SqlLib::pglib_GetErrorMessage(void)
{
	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return TEXT("");
	return this->db.pglib->GetErrorMessage();
}

TCHAR *		SqlLib::pglib_GetTypeName(SqlLib::VarBuffer & type, SqlLib::TableItem & item)
{
	TCHAR		tmp[256];

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return NULL;
	type.Clear();
	switch ( item.dtype )
	{
		case DTYPE_BOOLEAN:
			type.AddText(TEXT("boolean"));
			break;

		case DTYPE_SMALLINT:
			type.AddText(TEXT("smallint"));
			break;
		case DTYPE_INTEGER:
			type.AddText(TEXT("integer"));
			break;
		case DTYPE_BIGINT:
			type.AddText(TEXT("bigint"));
			break;
		case DTYPE_FLOAT:
			type.AddText(TEXT("float4"));
			break;
		case DTYPE_DOUBLE:
			type.AddText(TEXT("float8"));
			break;

		case DTYPE_CHAR:
			type.AddText(TEXT("char"));
			_stprintf(tmp, TEXT("(%d)"), item.size);
			type.AddText(tmp);
			break;
		case DTYPE_TEXT:
			type.AddText(TEXT("text"));
			break;
		case DTYPE_VARCHAR:
			type.AddText(TEXT("varchar"));
			_stprintf(tmp, TEXT("(%d)"), item.size);
			type.AddText(tmp);
			break;
		case DTYPE_BINARY:
			type.AddText(TEXT("bytea"));
			break;
		case DTYPE_VARBINARY:
			type.AddText(TEXT("bytea"));
			break;

		case DTYPE_DATE:
			type.AddText(TEXT("date"));
			break;
		case DTYPE_TIME:
			type.AddText(TEXT("time"));
			break;
		case DTYPE_TIMESTAMP:
			type.AddText(TEXT("timestamp"));
			break;

		case DTYPE_SERIAL:
			type.AddText(TEXT("serial"));
			break;
		case DTYPE_BIGSERIAL:
			type.AddText(TEXT("bigserial"));
			break;

		case DTYPE_USERTYPE:
			type.AddText(item.tname);
			break;

		default:
			return NULL;
	}
	type.AddNulChar();
	return type.GetText();
}

int		SqlLib::pglib_SetSqlValue(SqlLib::VarBuffer & str, SqlLib::TableItem & item, SqlLib::DataValue & val)
{
	TCHAR		tmp[256];

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	str.Clear();
	switch ( val.GetDataType() )
	{
		case DTYPE_NUL:
			str.AddText(TEXT("NUL"));
			break;

		case DTYPE_BOOLEAN:
			str.AddText( val.GetBoolean() ? TEXT("true") : TEXT("false") );
			break;

		case DTYPE_SMALLINT:
			_stprintf(tmp, TEXT("%d"), (int)val.GetSmallInt());
			str.AddText(tmp);
			break;
		case DTYPE_INTEGER:
			_stprintf(tmp, TEXT("%d"), (int)val.GetInteger());
			str.AddText(tmp);
			break;
		case DTYPE_BIGINT:
			_stprintf(tmp, TEXT("%"FI64"d"), (Int64)val.GetBigInt());
			str.AddText(tmp);
			break;
		case DTYPE_FLOAT:
			_stprintf(tmp, TEXT("%f"), (double)val.GetFloat());
			str.AddText(tmp);
			break;
		case DTYPE_DOUBLE:
			_stprintf(tmp, TEXT("%f"), (double)val.GetDouble());
			str.AddText(tmp);
			break;

		case DTYPE_CHAR:
		case DTYPE_TEXT:
		case DTYPE_VARCHAR:
		{
			if ( val.GetText() != NULL )
				SqlLib::EscapeString(str, (unsigned)_tcslen(val.GetText()), val.GetText(), TEXT('\''));
			else
				str.AddText(TEXT("NUL"));
			break;
		}
		case DTYPE_BINARY:
		case DTYPE_VARBINARY:
		{
			if ( val.GetBinary() != NULL )
				SqlLib::EscapeBinary(str, val.GetSize(), val.GetBinary(), TEXT('\''));
			else
				str.AddText(TEXT("NUL"));
			break;
		}

		case DTYPE_DATE:
			_stprintf(tmp, TEXT("'%04d-%02d-%02d'"), val.GetDate().year, val.GetDate().month, val.GetDate().day);
			str.AddText(tmp);
			break;
		case DTYPE_TIME:
			_stprintf(tmp, TEXT("'%02d:%02d:%02d'"), val.GetTime().hour, val.GetTime().minute, val.GetTime().second);
			str.AddText(tmp);
			break;
		case DTYPE_TIMESTAMP:
			_stprintf(tmp, TEXT("'%04d-%02d-%02d %02d:%02d:%02d'")
			  , val.GetTimeStamp().year, val.GetTimeStamp().month , val.GetTimeStamp().day
			  , val.GetTimeStamp().hour, val.GetTimeStamp().minute, val.GetTimeStamp().second
			);
			str.AddText(tmp);
			break;

		case DTYPE_SERIAL:
			_stprintf(tmp, TEXT("%d"), (int)val.GetSerial());
			str.AddText(tmp);
			break;
		case DTYPE_BIGSERIAL:
			_stprintf(tmp, TEXT("%"FI64"d"), (Int64)val.GetBigSerial());
			str.AddText(tmp);
			break;

		case DTYPE_USERTYPE:
			if( val.GetSize() > 0 && val.GetBinary() != NULL )
				str.Add(val.GetSize(), val.GetBinary());
			break;

		default:
			return -1;
	}
	str.AddNulChar();

	return 0;
}


SqlLib::ResultStatus	SqlLib::pglib_GetStatus(SqlLib::Result & res)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return RES_ERROR;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return RES_ERROR;
	switch ( pg_res->GetStatus() )
	{
	case PgLib::PGRES_COMMAND_OK: return RES_OK;
	case PgLib::PGRES_TUPLES_OK : return RES_OK;
	default:
		break;
	}
	return RES_ERROR;
}

int		SqlLib::pglib_GetTuples(SqlLib::Result & res)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	return pg_res->GetTuples();
}

int		SqlLib::pglib_GetFields(SqlLib::Result & res)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	return pg_res->GetFields();
}

TCHAR *		SqlLib::pglib_GetFname(SqlLib::Result & res, int idx)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return NULL;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return NULL;
	return pg_res->GetFname(idx);
}

int			SqlLib::pglib_GetFnumber(SqlLib::Result & res, const TCHAR * fname)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	return pg_res->GetFnumber(fname);
}

SqlLib::DataType	SqlLib::pglib_GetFtype(SqlLib::Result & res, int idx)
{
	PgLib::Result *		pg_res;
	PgLib::Oid			pg_oid;
	PgLib::PgTypeItem *	pg_typeitem;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return DTYPE_ERROR;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return DTYPE_ERROR;
	if ( (pg_oid = pg_res->GetFtype(idx)) < 0 )
		return DTYPE_ERROR;
	if ( (pg_typeitem = this->db.pglib->GetPgType(pg_oid)) == NULL )
		return DTYPE_UNKNOWN;
	switch ( pg_typeitem->type )
	{
	case PgLib::PGLIB_BOOL:	return DTYPE_BOOLEAN;

	case PgLib::PGLIB_INT2     : return DTYPE_SMALLINT;
	case PgLib::PGLIB_INT4     : return DTYPE_INTEGER;
	case PgLib::PGLIB_INT8     : return DTYPE_BIGINT;
	case PgLib::PGLIB_FLOAT4   : return DTYPE_FLOAT;
	case PgLib::PGLIB_FLOAT8   : return DTYPE_DOUBLE;

	case PgLib::PGLIB_CHAR     : return DTYPE_CHAR;
	case PgLib::PGLIB_VARCHAR  : return DTYPE_VARCHAR;
	case PgLib::PGLIB_TEXT     : return DTYPE_TEXT;
	case PgLib::PGLIB_BYTEA    : return DTYPE_VARBINARY;

	case PgLib::PGLIB_DATE     : return DTYPE_DATE;
	case PgLib::PGLIB_TIME     : return DTYPE_TIME;
	case PgLib::PGLIB_TIMESTAMP: return DTYPE_TIMESTAMP;
	}
	return DTYPE_UNKNOWN;
}


int		SqlLib::pglib_GetFmod(SqlLib::Result & res, int idx)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return DTYPE_ERROR;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return DTYPE_ERROR;
	return pg_res->GetFmod(idx);
}

int		SqlLib::pglib_GetFsize(SqlLib::Result & res, int idx)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return DTYPE_ERROR;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return DTYPE_ERROR;
	return pg_res->GetFsize(idx);
}

TCHAR *		SqlLib::pglib_GetValue(SqlLib::Result & res, int y, int x)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return NULL;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return NULL;
	return pg_res->GetValue(y, x);
}

int			SqlLib::pglib_GetDataValue(SqlLib::Result & res, int y, int x, SqlLib::DataValue & val)
{
	PgLib::Result *		pg_res;
	const TCHAR *		sval;
	DataType			dtype;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	if ( this->pglib_IsNull(res, y, x) > 0 )
		return 0;	// NUL
	if ( (sval = pg_res->GetValue(y, x)) == NULL )
		return -1;
	dtype = this->pglib_GetFtype(res, x);
	switch ( dtype )
	{
	case DTYPE_BOOLEAN:
		switch ( *sval )
		{
			case TEXT('T'):
			case TEXT('t'):
				val.SetBoolean(TRUE);
				return 1;
			case TEXT('F'):
			case TEXT('f'):
				val.SetBoolean(FALSE);
				return 1;
		}
		break;

	case DTYPE_SMALLINT:
		val.SetSmallInt((SmallInt)atoi(sval));
		return 1;
	case DTYPE_INTEGER:
		val.SetInteger((Integer) atoi(sval));
		return 1;
	case DTYPE_BIGINT:
		val.SetBigInt((BigInt) SqlLib::atoi64(sval));
		return 1;
	case DTYPE_FLOAT:
		val.SetFloat((Float) atof(sval));
		return 1;
	case DTYPE_DOUBLE:
		val.SetDouble((Double) atof(sval));
		return 1;

	case DTYPE_CHAR:
		val.SetChar((TCHAR *)sval, this->pglib_GetLength(res, y, x));
		return 1;
	case DTYPE_TEXT:
		val.SetText((TCHAR *)sval, this->pglib_GetLength(res, y, x));
		return 1;
	case DTYPE_VARCHAR:
		val.SetVarChar((TCHAR *)sval, this->pglib_GetLength(res, y, x));
		return 1;
	case DTYPE_BINARY:
		val.SetBinary((void *)sval, this->pglib_GetLength(res, y, x));
		return 1;
	case DTYPE_VARBINARY:
		val.SetVarBinary((void *)sval, this->pglib_GetLength(res, y, x));
		return 1;

	case DTYPE_DATE:
	{
		int				n, ar[3];
		SqlLib::Date	date;

		if ( (n = GetDateTimeValue(3, ar, sval)) < 3 )
			return -1;
		date.year  = ar[0];
		date.month = ar[1];
		date.day   = ar[2];
		val.SetDate(date);
		return 1;
	}
	case DTYPE_TIME:
	{
		int				n, ar[3];
		SqlLib::Time	time;

		if ( (n = GetDateTimeValue(3, ar, sval)) < 3 )
			return -1;
		time.hour   = ar[0];
		time.minute = ar[1];
		time.second = ar[2];
		val.SetTime(time);
		return 1;
	}
	case DTYPE_TIMESTAMP:
	{
		int					n, ar[6];
		SqlLib::TimeStamp	ts;

		if ( (n = GetDateTimeValue(6, ar, sval)) < 6 )
			return -1;
		ts.year   = ar[0];
		ts.month  = ar[1];
		ts.day    = ar[2];
		ts.hour   = ar[3];
		ts.minute = ar[4];
		ts.second = ar[5];
		val.SetTimeStamp(ts);
		return 1;
	}

	case DTYPE_SERIAL:
		val.SetSerial((Integer) atoi(sval));
		return 1;
	case DTYPE_BIGSERIAL:
		val.SetBigSerial((BigInt) SqlLib::atoi64(sval));
		return 1;

	default:
		return -1;
	}
	return 0;
}

int		SqlLib::pglib_IsNull(SqlLib::Result & res, int y, int x)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	return pg_res->GetIsNull(y, x);
}

int		SqlLib::pglib_GetLength(SqlLib::Result & res, int y, int x)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return -1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return -1;
	return pg_res->GetLength(y, x);
}

TCHAR *		SqlLib::pglib_GetCmdStatus(SqlLib::Result & res)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return NULL;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return NULL;
	return pg_res->GetCmdStatus();
}

SqlLib::Oid		SqlLib::pglib_GetOidValue(SqlLib::Result & res)
{
	PgLib::Result *		pg_res;

	if ( this->dbtype != DBTYPE_PGLIB || this->db.pglib == NULL )
		return (Oid)-1;
	if ( (pg_res = (PgLib::Result *)res.pglib_getres()) == NULL )
		return (Oid)-1;
	return pg_res->GetOidValue();
}

#endif
