//////////////////////////////////////////////////////////////////////////
//	"sqltest.cpp" - SqlLib test
//------------------------------------------------------------------------
//	$Id: sqltest.cpp 72 2006-03-28 10:52:06Z taro $
//////////////////////////////////////////////////////////////////////////

#include	<stdlib.h>
#include	<stdio.h>
#include	"sqllib.h"

#include	"../../include/sdb_tbldef.h"

namespace	sdb
{
SqlLib::TableItem	dev_tbl[DEV_FLD_NUM] =
{
	{ DEV_ID         , TEXT("dev_id"         ), SqlLib::DTYPE_SERIAL   , TEXT(""),   0, SqlLib::FF_SERIAL
	                                                                                  | SqlLib::FF_PRIMARY, TEXT("device-id"         )},
	{ DEV_MODEL      , TEXT("dev_model"      ), SqlLib::DTYPE_VARCHAR  , TEXT(""),  64, SqlLib::FF_INDEX  , TEXT("model"             )},
	{ DEV_SERIAL     , TEXT("dev_serial"     ), SqlLib::DTYPE_VARCHAR  , TEXT(""),  64, SqlLib::FF_INDEX  , TEXT("serial"            )},
	{ DEV_HOSTNAME   , TEXT("dev_hostname"   ), SqlLib::DTYPE_VARCHAR  , TEXT(""),  64, SqlLib::FF_INDEX  , TEXT("hostname"          )},
	{ DEV_GROUPNAME  , TEXT("dev_groupname"  ), SqlLib::DTYPE_VARCHAR  , TEXT(""),  64, SqlLib::FF_INDEX  , TEXT("groupname"         )},
	{ DEV_SMT_TBLNAME, TEXT("dev_smt_tblname"), SqlLib::DTYPE_TEXT     , TEXT(""),  64, 0                 , TEXT("table name"        )},
	{ DEV_F_DISABLE  , TEXT("dev_f_disable"  ), SqlLib::DTYPE_BOOLEAN  , TEXT(""),   0, 0                 , TEXT("disable"           )},
	{ DEV_BLOCKSIZE  , TEXT("dev_blocksize"  ), SqlLib::DTYPE_INTEGER  , TEXT(""),   0, 0                 , TEXT("block-size"        )},
	{ DEV_BLOCKNUMBER, TEXT("dev_blocknumber"), SqlLib::DTYPE_BIGINT   , TEXT(""),   0, 0                 , TEXT("block-number"      )},
	{ DEV_TS_BEGIN   , TEXT("dev_ts_begin"   ), SqlLib::DTYPE_TIMESTAMP, TEXT(""),   0, 0                 , TEXT("begin date-time"   )},
	{ DEV_TS_LAST    , TEXT("dev_ts_last"    ), SqlLib::DTYPE_TIMESTAMP, TEXT(""),   0, 0                 , TEXT("last date-time"    )},

	{ DEV_MEMO       , TEXT("dev_memo"       ), SqlLib::DTYPE_TEXT     , TEXT(""),   0, 0                 , TEXT("remark"            )},
};
} // end of namespace

int		main(int argc, char ** argv)
{
	static char			*conn_param = "host=venom dbname=sdb_debug user=sdb_user password=sdb_pass";
	SqlLib::ConnectInfo	conn_info;
	SqlLib				sqllib;
	SqlLib::Result		res;
	SqlLib::TimeStamp	ts;
	const char			*sql = NULL, *query = NULL;
	int					i, xmax, ymax, x, y;
	int					retcode;

	printf("***** sqllib test program *****\n");
	for ( i = 1; i < argc; i++ )
	{
		const char	*s;

		s = argv[i];
		if ( sql == NULL )
			sql = s;
	}

	try
	{
		if ( sqllib.SetDBType(SqlLib::DBTYPE_PGLIB) < 0 )
		{
			fprintf(stderr, "SetDBType Errpr");
			throw -1;
		}

		conn_info.host   = "venom";
		conn_info.port   = NULL;
		conn_info.dbname = "sdb_debug";
		conn_info.user   = "sdb_user";
		conn_info.pass   = "sbd_pass";
		conn_info.option = NULL;
		if ( sqllib.Connect(&conn_info) < 0 || sqllib.IsConnect() == FALSE )
		{
			fprintf(stderr, "Connect Error!\n");
			throw -1;
		}

		{
			SqlLib::TableDef	tdef;
			SqlLib::FieldsTbl	ftbl;
			SqlLib::VarBuffer	sql;

			for ( i = 0; i < sizeof(sdb::dev_tbl)/sizeof(*sdb::dev_tbl); i++ )
			{
				if ( tdef.Add(sdb::dev_tbl[i]) < 0 )
				{
					fprintf(stderr, "Add TableDef Error!\n");
					throw -1;
				}
			}
#if	0	// CREATE TABLE
			if ( sqllib.CreateTable(TEXT("test"), tdef) < 0 )
			{
				fprintf(stderr, "Create Table Error!\n");
				throw -1;
			}
#endif

			ftbl.AddVarChar  (sdb::dev_tbl[sdb::DEV_MODEL      ].fname, TEXT("HDD-MODEL" ));
			ftbl.AddVarChar  (sdb::dev_tbl[sdb::DEV_SERIAL     ].fname, TEXT("HDD-SERIAL"));
			ftbl.AddVarChar  (sdb::dev_tbl[sdb::DEV_HOSTNAME   ].fname, TEXT("venom"));
			ftbl.AddVarChar  (sdb::dev_tbl[sdb::DEV_GROUPNAME  ].fname, TEXT(""));
			ftbl.AddBoolean  (sdb::dev_tbl[sdb::DEV_F_DISABLE  ].fname, FALSE);
			ftbl.AddInteger  (sdb::dev_tbl[sdb::DEV_BLOCKSIZE  ].fname, 512);
			ftbl.AddBigInt   (sdb::dev_tbl[sdb::DEV_BLOCKNUMBER].fname, 32768);
			ts.year = 2006; ts.month = 3; ts.day = 7;
			ts.hour = ts.minute = ts.second = 0;
			ftbl.AddTimeStamp(sdb::dev_tbl[sdb::DEV_TS_BEGIN   ].fname, ts);
			ftbl.AddTimeStamp(sdb::dev_tbl[sdb::DEV_TS_LAST    ].fname, ts);
			ftbl.AddText     (sdb::dev_tbl[sdb::DEV_MEMO       ].fname, TEXT(""));

			sql.Clear();
			if ( sqllib.GetSql_Insert(sql, TEXT("test"), tdef, ftbl) < 0 )
			{
				fprintf(stderr, "GetSql_Insert Error!\n");
				throw -1;
			}
			printf("sql = %s\n", sql.GetText());

#if 0	// DROP TABLE
			if ( sqllib.DropTable(TEXT("test"), tdef) < 0 )
			{
				fprintf(stderr, "Drop Table Error!\n");
				throw -1;
			}
#endif
		}

		printf("BEGIN\n");
		if ( sqllib.Exec(res, "BEGIN") < 0 || sqllib.GetStatus(res) != SqlLib::RES_OK )
		{
			fprintf(stderr, "BEGIN command failed: %s", sqllib.GetErrorMessage());
			throw -1;
		}
		sqllib.ClearResult(res);

		query = "SELECT * FROM dev";
		printf("%s\n", query);
		if ( sqllib.Exec(res, query) < 0 || sqllib.GetStatus(res) != SqlLib::RES_OK )
		{
			fprintf(stderr, "%s: %s", query, sqllib.GetErrorMessage());
			throw -1;
		}

		if ( (xmax = sqllib.GetFields(res)) > 0 )
		{
			for ( x = 0; x < xmax; x++ )
			{
				const char			*fname;
				int					fsize, fmod;
				SqlLib::DataType	dtype;

				fname  = sqllib.GetFname(res, x);
				dtype  = sqllib.GetFtype(res, x);
				fsize  = sqllib.GetFsize(res, x);
				fmod   = sqllib.GetFmod(res, x);
				if ( fname != NULL )
					printf("%i:%s dtype:%d (%d:%d)\n", x, fname, (int)dtype, fsize, fmod);
			}
			printf("\n");

			if ( (ymax = sqllib.GetTuples(res)) > 0  )
			{
				for ( y = 0; y < ymax; y++ )
				{
					for ( x = 0; x < xmax; x++ )
					{
						int		length;

						length = sqllib.GetLength(res, y, x);
						printf("\t(%d)", length);
						if ( sqllib.IsNull(res, y, x)  )
						{
							printf("NUL");
						} else
						{
							const char	*fval;

							fval = sqllib.GetValue(res, y, x);
							printf("%s", fval);
						}
					}
					printf("\n");
				}
			}
		}
		sqllib.ClearResult(res);

		/* end the transaction */
		printf("END\n");
		sqllib.Exec(res, "END");
		sqllib.ClearResult(res);
	} catch(int code) { retcode = code;	}
	printf("finish\n");
	sqllib.Finish();

	return 0;
}

