//////////////////////////////////////////////////////////////////////////
//	"sdb_file.cpp"
//------------------------------------------------------------------------
//	$Id: sdb_file.cpp 149 2006-04-30 04:51:21Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"../include/sdblib.h"
#include	"../lib/varbuf.h"
#if	defined(WIN32)
# include	<sqpath.h>
# include	<crtdbg.h>
#else
# include	"../linux/defs.h"
# include	"../lib/sqpath.h"
# include	<stdio.h>
# include	<unistd.h>
# include	<fcntl.h>
# include	<sys/ioctl.h>
# include	<sys/stat.h>
#endif

#if	defined(_DEBUG) && defined(WIN32)
#	define	DBGREPORT(param_)	_CrtDbgReport param_
#else
#	define	DBGREPORT(param_)
# ifndef	_ASSERT
#	define	_ASSERT
# endif
#endif

namespace	sdb
{
#if	defined(WIN32)
const FileHandle	Invalid_FileHandle_Value = NULL;
#elif	defined(LINUX)
const FileHandle	Invalid_FileHandle_Value = -1;
#else
const FileHandle	Invalid_FileHandle_Value = NULL;
#endif

BOOL		IsFileHandleValid(FileHandle handle)
{
#if	defined(WIN32)
	return handle != NULL && handle != INVALID_HANDLE_VALUE;
#elif	defined(LINUX)
	return handle >= 0;
#else
	return handle != NULL;
#endif
}

FileHandle	FileOpen(const TCHAR * path, FileOpenMode mode)
{
	FileHandle	handle;

#if	defined(WIN32)
	{
		DWORD	dwAcc = 0;
		DWORD	dwShare = 0;
		DWORD	dwCreate = 0;

		switch ( mode )
		{
		case FOM_READ:
			dwAcc    = GENERIC_READ;
			dwShare  = FILE_SHARE_READ;
			dwCreate = OPEN_EXISTING;
			break;

		case FOM_WRITE:
			dwAcc = GENERIC_READ | GENERIC_WRITE;
			dwShare = 0;
			dwCreate = CREATE_ALWAYS;
			break;

		case FOM_APPEND:
			dwAcc = GENERIC_READ | GENERIC_WRITE;
			dwShare = 0;
			dwCreate = OPEN_ALWAYS;
			break;
		default:
			return NULL;
		}
		handle = ::CreateFile(path, dwAcc, dwShare, NULL, dwCreate, FILE_ATTRIBUTE_NORMAL, NULL);
		if ( handle == INVALID_HANDLE_VALUE )
			handle = NULL;
	}
#elif	defined(LINUX)
	{
		int		flags = 0;

		switch ( mode )
		{
		case FOM_READ:
			flags = O_RDONLY;
			break;

		case FOM_WRITE:
			flags = O_RDWR | O_CREAT | O_TRUNC;
			break;

		case FOM_APPEND:
			flags = O_RDWR | O_CREAT | O_APPEND;
			break;
		default:
			return -1;
		}
		if ( (handle = open(path, flags)) == -1 )
			handle = -1;
	}
#else
	{
		const TCHAR	*mode_text;

		switch ( mode )
		{
		case FOM_READ:
			mode_text = TEXT("rb");
			break;

		case FOM_WRITE:
			mode_text = TEXT("wb");
			break;

		case FOM_APPEND:
			mode_text = TEXT("ab");
			break;
		default:
			return NULL;
		}
		handle = _tfopen(path, mode_text);
	}
#endif
	return handle;
}

int		FileClose(FileHandle handle)
{
	if ( IsFileHandleValid(handle) == FALSE )
		return 0;
#if	defined(WIN32)
	if ( handle == INVALID_HANDLE_VALUE )
		return 0;
	if ( ::CloseHandle(handle) == 0 )
		return -1;
#elif	defined(LINUX)
	if ( close(handle) == -1 )
		return -1;
#else
	if ( fclose(handle) == EOF )
		return -1;
#endif
	return 0;
}

int			FileRead(FileHandle handle, unsigned size, void * buffer)
{
	int		ret;

	if ( IsFileHandleValid(handle) == FALSE )
		return -1;
#if	defined(WIN32)
	{
		DWORD	dw;

		if ( ::ReadFile(handle, buffer, size, &dw, NULL) == 0 )
			return -1;
		ret = (int)dw;
	}
#elif	defined(LINUX)
	ret = (int)read(handle, buffer, size);
#else
	ret = (int)fread(buffer, 1, size, handle);
#endif
	return ret;
}

int			FileWrite(FileHandle handle, unsigned size, const void * buffer)
{
	int		ret;

	if ( IsFileHandleValid(handle) == FALSE )
		return -1;
#if	defined(WIN32)
	{
		DWORD	dw;

		if ( ::WriteFile(handle, buffer, size, &dw, NULL) == 0 )
			return -1;
		ret = (int)dw;
	}
#elif	defined(LINUX)
	ret = (int)write(handle, buffer, size);
#else
	ret = (int)fwrite(buffer, 1, size, handle);
#endif
	return ret;
}

Int64		FileSeek(FileHandle handle, Int64 offset, int pos)
{
	if ( IsFileHandleValid(handle) == FALSE )
		return I64(-1);
#if	defined(WIN32)
	{
		LARGE_INTEGER	move;

		move.QuadPart = offset;
		if ( ::SetFilePointerEx(handle, move, NULL, pos) == 0 )
			return I64(-1);
	}
#elif	defined(LINUX)
	if ( lseek(handle, (off_t)offset, pos) < 0 )
		return I64(-1);
#else
	if ( _fseeki64(handle, offset, pos) != 0 )
		return I64(-1);
#endif
	return I64(0);
}

};// end of namespace sdb;

const TCHAR *		SdbLib::GetSubPath(SQStr & path)
{
	SQPath	str;

	str.SetText(sdb::sub_dir);
	str.AddPath(SDB_LOG_FILENAME);
	path = str;
	return path.GetText();
}

const TCHAR *		SdbLib::GetLogText(SQStr & text, sdb::DBDevData & devdata, sdb::DBSmtData & smtdata)
{
	VarBuffer			vbuf;

	vbuf.Printf(TEXT("%4d-%02d-%02d %02d:%02d:%02d")
	  , smtdata.smt_ts.year, smtdata.smt_ts.month, smtdata.smt_ts.day
	  , smtdata.smt_ts.hour, smtdata.smt_ts.minute, smtdata.smt_ts.second
	);
#if 0
	vbuf.AddChar(TEXT(','));
	vbuf.AddText(devdata.dev_model.GetDisplayText());
	vbuf.AddChar(TEXT(','));
	vbuf.AddText(devdata.dev_serial.GetDisplayText());
#else
	{
		SqlLib::VarBuffer	sqlbuf;

		sqlbuf.Clear();
		SqlLib::EscapeString(sqlbuf, devdata.dev_model.GetDisplayText(), TEXT('\''));
		sqlbuf.AddNulChar();
		vbuf.AddChar(TEXT(','));
		vbuf.AddText(sqlbuf.GetText());

		sqlbuf.Clear();
		SqlLib::EscapeString(sqlbuf, devdata.dev_serial.GetDisplayText(), TEXT('\''));
		sqlbuf.AddNulChar();
		vbuf.AddChar(TEXT(','));
		vbuf.AddText(sqlbuf.GetText());
	}
#endif

	vbuf.AddChar(TEXT(','));
	vbuf.Printf(TEXT("%u,%u,%u,%u"), smtdata.smt_attr_id, smtdata.smt_value, smtdata.smt_worst, smtdata.smt_threshold);
	vbuf.AddChar(TEXT(','));
	vbuf.Printf(TEXT("%u"), smtdata.smt_flags);
	vbuf.AddChar(TEXT(','));
	vbuf.Printf(TEXT("%"FI64"u,%"FI64"u"), smtdata.smt_raw_value, sdb::ATA_GET48(smtdata.smt_raw_data));


#if	defined(WIN32)
	vbuf.AddText(TEXT("\r\n"));
#else
	vbuf.AddText(TEXT("\n"));
#endif
	vbuf.AddChar(TEXT('\0'));
	text.SetText(vbuf.GetText());
	return text.GetText();
}

