//////////////////////////////////////////////////////////////////////////
//	"sdb_iobuf.cpp"
//------------------------------------------------------------------------
//	$Id: sdb_iobuf.cpp 151 2006-04-30 23:09:32Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"../include/sdblib.h"
#include	"../lib/varbuf.h"
#include	<string.h>
#include	<stdio.h>
#if	defined(WIN32)
# include	<crtdbg.h>
#endif

#if	defined(_DEBUG) && defined(WIN32)
#	define	DBGREPORT(param_)	_CrtDbgReport param_
#elif	defined(DEBUG) && defined(LINUX)
#	define	_ASSERT	assert
#	define	DBGREPORT(param_)	DbgReport param_
#	define	_CRT_WARN			0
	extern int		DbgReport(int type, const char *filename, int linenumber, const char *moduleName, const char *format, ... );
#else
#	define	DBGREPORT(param_)
# ifndef	_ASSERT
#		define	_ASSERT
# endif
#endif

namespace	sdb {

IOBuffer::IOBuffer()
{
	this->mode = FOM_NOTHING;
	this->buffersize = 0;
	this->validsize = 0;
	this->pos = 0;
	this->buffer = NULL;
	this->hFile = Invalid_FileHandle_Value;
	this->unget_level = 0;
	this->filename = NULL;
	this->line = 0;
	this->col = 0;
}

IOBuffer::~IOBuffer()
{
	this->CloseFile();
	if ( this->buffer != NULL )
	{
		delete [] this->buffer;
		this->buffer = NULL;
	}
	if ( this->filename != NULL )
	{
		delete [] this->filename;
		this->filename = NULL;
	}
}

int		IOBuffer::CreateBuffer(unsigned size)
{
	if ( size == 0 )
		return -1;
	if ( this->hFile != Invalid_FileHandle_Value )
		return -1;
	if ( this->buffer != NULL )
		delete [] this->buffer;
	if ( (this->buffer = new BYTE [size]) == NULL )
		return -1;
	this->buffersize  = size;
	this->validsize   = 0;
	this->pos         = 0;
	this->unget_level = 0;
	this->line = 0;
	this->col = 0;

	return 0;
}

int		IOBuffer::OpenFile(const TCHAR * path, FileOpenMode mode)
{
	_ASSERT(path != NULL);
	if ( this->hFile != Invalid_FileHandle_Value )
		return -1;
	if ( this->buffer == NULL )
	{
		if ( this->CreateBuffer() < 0 )
			return -1;
	}
	if ( this->filename != NULL )
	{
		unsigned	len;

		delete [] this->filename;
		len = (unsigned) _tcslen(path);
		if ( (this->filename = new TCHAR [len+1]) == NULL )
			return -1;
		_tcscpy(this->filename, path);
	}
	if ( (this->hFile = FileOpen(path, mode)) == Invalid_FileHandle_Value  )
		return -1;

	this->mode = mode;
	this->validsize   = 0;
	this->pos         = 0;
	this->unget_level = 0;
	this->line = 0;
	this->col = 0;

	return 0;
}

int		IOBuffer::CloseFile(void)
{
	if ( this->hFile != Invalid_FileHandle_Value )
	{
		FileClose(this->hFile);
		this->hFile = Invalid_FileHandle_Value ;
	}
	return 0;
}


int		IOBuffer::ReadBuffer(void)
{
	int		size;

	if ( this->hFile == Invalid_FileHandle_Value || this->buffer == NULL )
		return -1;
	if ( (size = FileRead(this->hFile, this->buffersize, this->buffer)) < 0 )
		return RES_ERROR;
	this->validsize = size;
	return (int) this->validsize;
}

BOOL		IOBuffer::IsEnd(void)
{
	if ( this->unget_level > 0 )
		return FALSE;
	if ( this->pos >= this->validsize )
	{
		if ( this->ReadBuffer() < 0 || this->validsize == 0 )
			return TRUE;
		this->pos = 0;
	}
	return FALSE;
}

int		IOBuffer::UnGetChar(int ch)
{
	if ( ch < 0 )
		return -1;
	if ( this->unget_level >= UNGET_MAX )
		return -1;
	this->unget_ch[this->unget_level++] = ch;
	return 0;
}

int		IOBuffer::GetChar(void)
{
	int		ch = 0, ch2;
	int		loop = 0;

	if ( this->unget_level > 0 )
	{
		this->unget_level--;
		return this->unget_ch[this->unget_level];
	}
	for (;;)
	{
		if ( this->pos >= this->validsize )
		{
			int		res;

			if ( (res = this->ReadBuffer()) < 0 )
				return res;
			if (  this->validsize == 0 )
				return RES_EOF;
			this->pos = 0;
		}
		this->col++;
		ch = this->buffer[this->pos++];
		if ( loop == 0 )
		{
			if ( _ismbblead(ch) == FALSE )
				break;
			ch2 = ch;
			loop = 1;
			continue;
		} else
		{
			if ( _ismbbtrail(ch) )
				ch = (ch2 << 8) | ch;
			else
			{
				this->pos--;
				this->col--;
			}
			break;
		}
	}
	if ( ch == '\n' )
	{
		this->line++;
		this->col = 0;
	}
	return ch;
}

int		IOBuffer::SkipSpace(void)
{
	int			ch;

	for (;;)
	{
		if ( (ch = this->GetChar()) < 0 )
			return 0;
		switch ( ch )
		{
			case TEXT(' '):
			case TEXT('\t'):
			case TEXT('\r'):
			case TEXT('\n'):
				break;
			default:
				this->UnGetChar(ch);
				return 0;
		}
	}
}

int		IOBuffer::GetLine(VarBuffer & line)
{
	int			ch;
	unsigned	size = 0;

	if ( (ch = this->GetChar()) < 0 )
		return ch;
	for(;;)
	{
		if ( ch == TEXT('\n') )
			break;
		if ( ch != TEXT('\r') )
		{
			if ( line.AddChar(ch) < 0 )
				return RES_ERROR;
			size++;
		}
		if ( (ch = this->GetChar()) < 0 )
			break;
	}
	if ( line.AddChar('\0') < 0 )
		return RES_ERROR;

	return size;
}

} // end of namespace sdb

#if	defined(SDB_IOBUF_DEBUG)
int		main(int argc, char **argv)
{
	sdb::IOBuffer		iobuf;
	int					i, ch;
	char				*path = NULL;

	iobuf.CreateBuffer(256);
	for ( i = 1; i< argc; i++ )
	{
		const char	*s;

		s = argv[i];
		if ( path == NULL )
			path = (char *)s;
	}
	if ( path == NULL )
		return 1;
	fprintf(stdout, "OpenFile %s\n", path);
	if ( iobuf.OpenFile(path, sdb::FOM_READ) < 0 )
	{
		fprintf(stderr, "OpenFile Error (%s)\n", path);
		return 1;
	}
	while ( (ch = iobuf.GetChar()) >= 0 )
	{
		fputc(ch, stdout);
	}
	iobuf.CloseFile();

	return 0;
}
#endif
