//////////////////////////////////////////////////////////////////////////
//	"sdb_smartattr.h"
//------------------------------------------------------------------------
//	$Id: sdb_smartattr.cpp 93 2006-04-15 16:08:33Z taro $
//////////////////////////////////////////////////////////////////////////

#include	"../include/sdblib.h"
#if	defined(WIN32)
# include	"../win32/devctl.h"
# include	<crtdbg.h>
#elif	defined(LINUX)
# include	"../linux/devctl.h"
#endif

#if	defined(_DEBUG) && defined(WIN32)
#	define	DBGREPORT(param_)	_CrtDbgReport param_
#else
#	define	DBGREPORT(param_)
# ifndef	_ASSERT
#	define	_ASSERT
# endif
#endif

namespace	sdb
{

SmartAttrList::SmartAttrList()
{
}

SmartAttrList::~SmartAttrList()
{
	if ( this->list.size() > 0 )
		this->clear();
}

void		SmartAttrList::push_back(SmartAttrItem * item)
{
	this->list.push_back(item);
	return;
}


void	SmartAttrList::clear(void)
{
	List::iterator		it;
	SmartAttrItem *		item;

	for ( it = this->list.begin(); it != this->list.end(); it++ )
	{
		item = *it;
		if ( item != NULL )
			delete item;
	}
	this->list.clear();
	return;
}

int			SmartAttrList::AddDef(const TCHAR * param)
{
	static const TCHAR			*sep = TEXT(":");
	TCHAR						*buffer = NULL, *token;
	unsigned					id = 0;
	SmartAttrRawType			type = SMART_RAW_NORMAL;
	TCHAR						*desc = NULL;
	SmartAttrItem				*attr_item;
	int							ret = 0;

	_ASSERT(param != NULL);
	if ( (buffer = new TCHAR [_tcslen(param) + 1]) == NULL )
		return -1;
	_tcscpy(buffer, param);
	if ( (token = _tcstok( buffer, sep)) != NULL )
		id = _ttoi(token);
	if ( id == 0 || id > 255 )
		ret = -1;
	if ( ret == 0 )
	{
		if ( (token = _tcstok( NULL, sep)) != NULL )
		{
			if ( _tcsicmp(token, TEXT("normal")) == 0 )
			{
				type = SMART_RAW_NORMAL;
			} else if ( _tcsicmp(token, TEXT("divword")) == 0 )
			{
				type = SMART_RAW_DIVWORD;
			} else if ( _tcsicmp(token, TEXT("dumphex")) == 0 )
			{
				type = SMART_RAW_DUMPHEX;
			} else if ( _tcsicmp(token, TEXT("word0")) == 0 )
			{
				type = SMART_RAW_WORD0;
			} else if ( _tcsicmp(token, TEXT("word1")) == 0 )
			{
				type = SMART_RAW_WORD1;
			} else if ( _tcsicmp(token, TEXT("word2")) == 0 )
			{
				type = SMART_RAW_WORD2;
			} else
			{
				ret = -1;
			}
		}
	}
	if ( ret == 0 )
	{
		if ( (token = _tcstok( NULL, sep)) != NULL )
			desc = token;
		if ( (attr_item = new SmartAttrItem) == NULL )
			ret = -1;
	}
	if ( ret == 0 )
	{
		attr_item->id   = id;
		attr_item->type = type;
		attr_item->text.SetText(desc);
		this->push_back(attr_item);
	}
	if ( buffer != NULL )
		delete [] buffer;

	return ret;
}

SmartAttrItem *	SmartAttrList::GetSmartAttrItem(unsigned id)
{
	List::iterator		it;
	SmartAttrItem		*item = NULL;

	for ( it = this->list.begin(); it != list.end(); it++ )
	{
		if ( (*it)->id == id )
		{
			item = *it;
			break;
		}
	}
	return item;
}

const TCHAR * SmartAttrList::GetSmartRawValue(DevCtl::AtaSmartAttribute & attr, unsigned length, TCHAR * buffer)
{
	SmartAttrItem			*item;
	SmartAttrRawType		rawtype = SMART_RAW_NORMAL;
	Uint64					n64;
	TCHAR					tmpbuf[80];

	if ( (item = this->GetSmartAttrItem(attr.id)) != NULL )
		rawtype = item->type;
	n64 = sdb::ATA_GET48(attr.raw);
	switch ( rawtype )
	{
	case SMART_RAW_DUMPHEX:
		Fmt(tmpbuf, TEXT("%02X %02X %02X %02X %02X %02X "), attr.raw[0], attr.raw[1], attr.raw[2], attr.raw[3], attr.raw[4], attr.raw[5]);
		break;

	case SMART_RAW_DIVWORD:
	case SMART_RAW_WORD0:
	case SMART_RAW_WORD1:
	case SMART_RAW_WORD2:
	{
		unsigned	n1, n2, n3;

		n1 = sdb::ATA_GET16(&attr.raw[0]);
		n2 = sdb::ATA_GET16(&attr.raw[2]);
		n3 = sdb::ATA_GET16(&attr.raw[4]);
		Fmt(tmpbuf, TEXT("%6,u %6,u %6,u"), n1, n2, n3);
		break;
	}

	case SMART_RAW_NORMAL:
	default:
		Fmt(tmpbuf, TEXT("%20,I64u"), n64);
		break;
	}
	_tcsncpy(buffer, tmpbuf, length);

	return buffer;
}

sdb::Int64	SmartAttrList::GetSmartRawValue(DevCtl::AtaSmartAttribute & attr)
{
	SmartAttrItem			*item;
	SmartAttrRawType		rawtype = SMART_RAW_NORMAL;
	Uint64					n64;
	unsigned				w0, w1, w2;

	if ( (item = this->GetSmartAttrItem(attr.id)) != NULL )
		rawtype = item->type;
	n64 = sdb::ATA_GET48(attr.raw);
	w0 = sdb::ATA_GET16(&attr.raw[0]);
	w1 = sdb::ATA_GET16(&attr.raw[2]);
	w2 = sdb::ATA_GET16(&attr.raw[4]);
	switch ( rawtype )
	{
	case SMART_RAW_DUMPHEX:
		break;

	case SMART_RAW_DIVWORD:
	case SMART_RAW_WORD0:
		n64 = w0;
		break;
	case SMART_RAW_WORD1:
		n64 = w1;
		break;
	case SMART_RAW_WORD2:
		n64 = w2;
		break;
		break;

	case SMART_RAW_NORMAL:
	default:
		break;
	}

	return n64;
}


const TCHAR * GetSmartAttributeIdName(SmartVendor vendor, unsigned id, unsigned length, TCHAR * buffer)
{
	// MAX length 30
	static struct
	{
		unsigned		id;
		SmartVendor		vendor;
		const TCHAR		*text;
	} tbl[] =
	{
		{   1, SMART_VENDOR_GENERAL, TEXT("Raw Read Error Rate"           ) },
		{   2, SMART_VENDOR_GENERAL, TEXT("Throughput Performance"        ) },
		{   3, SMART_VENDOR_GENERAL, TEXT("Spin Up Time"                  ) },
		{   4, SMART_VENDOR_GENERAL, TEXT("Start/Stop Count"              ) },
		{   5, SMART_VENDOR_GENERAL, TEXT("Reallocated Sectors Count"     ) },
		{   6, SMART_VENDOR_GENERAL, TEXT("Read Channel Margin"           ) },
		{   7, SMART_VENDOR_GENERAL, TEXT("Seek Error Rate"               ) },
		{   8, SMART_VENDOR_GENERAL, TEXT("Seek Time Performance"         ) },
		{   9, SMART_VENDOR_GENERAL, TEXT("Power-On Hours"                ) },
		{  10, SMART_VENDOR_GENERAL, TEXT("Spin Retry Count"              ) },
		{  11, SMART_VENDOR_GENERAL, TEXT("Recalibration Retries"         ) },
		{  12, SMART_VENDOR_GENERAL, TEXT("Device Power Cycle Count"      ) },
		{  13, SMART_VENDOR_GENERAL, TEXT("Soft Read Error Rate"          ) },
		{ 191, SMART_VENDOR_GENERAL, TEXT("Gsense Error Rate"             ) },
		{ 192, SMART_VENDOR_GENERAL, TEXT("Power off Retract Count"       ) },
	//	{ 192, SMART_VENDOR_FUJITSU, TEXT("Emergency Retract Cycles"      ) },	// Fujitsu
		{ 193, SMART_VENDOR_GENERAL, TEXT("Load/Unload Cycle Count"       ) },
		{ 194, SMART_VENDOR_GENERAL, TEXT("Temperature"                   ) },
		{ 195, SMART_VENDOR_GENERAL, TEXT("Hardware ECC Recovered"        ) },
		{ 196, SMART_VENDOR_GENERAL, TEXT("Reallocation Event Count"      ) },
		{ 197, SMART_VENDOR_GENERAL, TEXT("Current Pending Sector Count"  ) },
		{ 198, SMART_VENDOR_GENERAL, TEXT("Uncorrectable Sector Count"    ) },
		{ 199, SMART_VENDOR_GENERAL, TEXT("UltraDMA CRC Error Rate"       ) },
		{ 200, SMART_VENDOR_WD     , TEXT("Multi Zone Error Rate"         ) },	// Western Digital
		{ 200, SMART_VENDOR_FUJITSU, TEXT("Write Error Rate"              ) },	// Fujitsu
		{ 201, SMART_VENDOR_GENERAL, TEXT("Soft Read Error Rate"          ) },
		{ 202, SMART_VENDOR_MAXTOR , TEXT("Data Address Mark Errors"      ) },	// Maxtor
		{ 202, SMART_VENDOR_FUJITSU, TEXT("TA Increase Count"             ) },	// Fujitsu
		{ 203, SMART_VENDOR_MAXTOR , TEXT("ECC Errors"                    ) },	// Maxtor
		{ 203, SMART_VENDOR_FUJITSU, TEXT("Run Out Cancel"                ) },	// Fujitsu
		{ 204, SMART_VENDOR_MAXTOR , TEXT("Soft ECC Correction"           ) },	// Maxtor
		{ 204, SMART_VENDOR_FUJITSU, TEXT("Shock Count Write Opern"       ) },	// Fujitsu
		{ 205, SMART_VENDOR_MAXTOR , TEXT("Thermal Aspirates"             ) },	// Maxtor
		{ 205, SMART_VENDOR_FUJITSU, TEXT("Shock Rate Write Opern"        ) },	// Fujitsu
		{ 206, SMART_VENDOR_FUJITSU, TEXT("Flying_Height"                 ) },	// Fujitsu
		{ 207, SMART_VENDOR_MAXTOR , TEXT("Spin High Current"             ) },	// Maxtor
		{ 208, SMART_VENDOR_MAXTOR , TEXT("Spin Buzz"                     ) },	// Maxtor
		{ 209, SMART_VENDOR_MAXTOR , TEXT("Offline Seek Performnce"       ) },	// Maxtor
		{ 220, SMART_VENDOR_GENERAL, TEXT("Disk Shift"                    ) },
		{ 221, SMART_VENDOR_GENERAL, TEXT("G-Sense Error Rate"            ) },
		{ 222, SMART_VENDOR_GENERAL, TEXT("Loaded Hours"                  ) },
		{ 223, SMART_VENDOR_GENERAL, TEXT("Load/Unload Retry Count"       ) },
		{ 224, SMART_VENDOR_GENERAL, TEXT("Load Friction"                 ) },
		{ 225, SMART_VENDOR_GENERAL, TEXT("Load Cycle Count"              ) },
		{ 226, SMART_VENDOR_GENERAL, TEXT("Load-in Time"                  ) },
		{ 227, SMART_VENDOR_GENERAL, TEXT("Torque Amplification Count"    ) },
		{ 228, SMART_VENDOR_GENERAL, TEXT("Power-off Retract Count"       ) },
		{ 230, SMART_VENDOR_GENERAL, TEXT("GMR Head Amplitude"            ) },	// IBM
		{ 231, SMART_VENDOR_GENERAL, TEXT("Temperature Celsius"           ) },
		{ 240, SMART_VENDOR_GENERAL, TEXT("Head Flying Hours"             ) },
		{ 250, SMART_VENDOR_GENERAL, TEXT("Read Error Retry Rate"         ) },
	//	{   0, SMART_VENDOR_GENERAL, TEXT(""                              ) },
	};
	int				i;
	const TCHAR		*text = NULL;

	for ( i = 0; i < sizeof(tbl)/sizeof(*tbl); i++ )
	{
		if ( tbl[i].id == id )
		{
			if ( text == NULL )
				text = tbl[i].text;
			if ( tbl[i].vendor == vendor )
				break;
			break;
		}
	}
	if ( text == NULL )
	{
		static TCHAR	msgbuf[80];

		_stprintf(msgbuf, TEXT("Unknown ID#%3d (%02XH)"), id, id);
		text = msgbuf;
	}
	if ( length > 0 && buffer != NULL )
	{
		if ( length > sdb::SMART_ATTRIBUTE_MAX_LENGHTH )
			length = sdb::SMART_ATTRIBUTE_MAX_LENGHTH;
		_tcsncpy(buffer, text, length);
		return buffer;
	} else
	{
		return text;
	}
}


};// end of namespace sdb;

