//////////////////////////////////////////////////////////////////////////
//	"sdbwatch.h"
//------------------------------------------------------------------------
//	$Id: sdbwatch.h 98 2006-04-19 06:09:44Z taro $
//////////////////////////////////////////////////////////////////////////
#ifndef	SDBWATCH_H_INCLUDED
#define	SDBWATCH_H_INCLUDED

#include	<sdblib.h>
#if	defined(SDBWATCH) || defined(LINUX)
# include	"../lib/sqpath.h"
#else
# include	<sqpath.h>
#endif
#include	"../lib/varbuf.h"
#include	<list>

#define	SDBWATCH_VERSION	SDB_GENERAL_VERSION
#define	SDBWATCH_TITLE		TEXT("***** sdbwtach - SDB database watch tool : ") SDBWATCH_VERSION TEXT(" *****")

class	SdbLib;
class	ScrLib;
class	SdbHttp;

class	SdbWatch
{
public:
	enum	CtlCmd
	{
		CMD_NOTHING   ,
		CMD_HELP      ,
		CMD_WATCH     ,
	};
	enum	ConsoleColumn
	{
		COLUMN1            = 38,
		COLUMN2            = 49,
		COLUMN3            = 60,
		COLUMN_MAX         = 80,
	};
	enum	CheckMode
	{
		CHECK_FAST,
		CHECK_LONG
	};

	SdbLib *		sdblib;
	sdb::DBWatLib *	dbwat;

	SQPath			progPath;
	SQPath			progDir;
	SQPath			defFile;
	BOOL			print_title;

	struct opt_tag
	{
		CheckMode	checkmode;
		int			offline_margin_sec;
		int			prev_time_sec;
		BOOL		disable_title;
		BOOL		sendmail;
	} opt;

public:
	class	WatchDev : public sdb::DBDevData
	{
	public:
		sdb::DBDevData	devdata;
	public:
		WatchDev();
		~WatchDev();
	};
	typedef std::list<WatchDev *>	WatchDevList;
	WatchDevList					watchDevList;
public:
//	WatchDev						*curWatchDev;

public:
	class AttrData
	{
	public:
		unsigned		count;
		sdb::DBSmtData	**smtlist;
		struct watchwork_tag
		{
			unsigned		count;
			sdb::DBWatData	watdata;
		} ww[sdb::WATCHTYPE_NUM];
	public:
		AttrData();
		~AttrData();

		int		CreateList(void);
	};
	class AttrMap
	{
	public:
		unsigned	attr_num;
		AttrData	id[SDB_SMART_ATTR_ID_MAX];
	};

protected:
	unsigned		watnum;
	sdb::DBWatData	*watlist;
	VarBuffer		watmsg;
	int				warn[sdb::WATCHWARN_NUM];

public:
	SdbWatch();
	~SdbWatch();

	void		PrintTitle(void);
	int			Init(void);
	int			LoadDef(void);
	int			CommandLine(int argc, const TCHAR ** argv);
	void		ClearWatchDevList(void);
	int			LocalDevice(void);
	int			LocalDevice(SdbDev * dev);
	int			Watch(void);
	int			GetWatchConf(void);
	int			WatchDevice(WatchDev & watdev);
	int			WatchWarn(sdb::Int64 & left_value, sdb::DBWatData & watdata, sdb::Int64 right_value);
	void		WarnClear(void);
	void		WarnCheck(sdb::DBDevData & devdata);
	int			GetWatValue(sdb::Int64 & value, sdb::WatchType wat_type, sdb::DBSmtData & smtdata);
	BOOL		LeftOpRight(sdb::Int64 left, const TCHAR * op, sdb::Int64 right);
	int			AnalysAttributeValue(unsigned attr_id, SdbWatch::AttrData & attrData, SqlLib::TimeStamp & ts_base);
	int			AnalysRawValue(unsigned attr_id, sdb::Int64 raw_value, SdbWatch::AttrData & attrData, SqlLib::TimeStamp & ts_base);
	void		PrintSmartData(sdb::DBDevData & devdata, sdb::DBSmtData & smtdata);
	int			WatchMessage(const TCHAR * text);
	void		PutPos(int pos, ... );
	static int	GetAttributeNum(AttrMap & attrmap, unsigned num, sdb::DBSmtData * smtlist);
	int			SendMail(sdb::DBDevData & devdata, const TCHAR * body_text);
	static const TCHAR *	GetDayTime(const SqlLib::TimeStamp & st, SQStr & str);
};

extern SdbWatch	sdbWatch;

#endif
